/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2005/11/28 21:59:11 $
 */
 
package kaos.core.csi;

import ri.JasBeanImpl;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;



/**
 * Defines the operations to store and access information about a capability an agent offers.
 *
 * @author KAoS Team
 * $Revision: 1.6 $
 */
public class AgentCapabilityImpl
    extends JasBeanImpl
    implements AgentCapability
{
    /**
	 * 
	 */
	private static final long serialVersionUID = 1315710359488866365L;
	
	private static final String COST = "kaos.core.csi.AGENT-CAPABILITY-COST";
    private static final String KEYWORDS = "kaos.core.csi.AGENT-CAPABILITY-KEYWORDS";
    private static final String NAME = "kaos.core.csi.AGENT-CAPABILITY-NAME";
    private static final String RELIABILITY = "kaos.core.csi.AGENT-CAPABILITY-RELIABILITY";
    private static final String WORDS_TO_REQUEST_CAPABILITY = "kaos.core.csi.AGENT-CAPABILITY-WORDS-TO-REQUEST-INFO";
    
    //AgentCapabilityImpl vars
     /*
        Determine if the List of requested capabilities matches the List of actual capabilities.
        As long as the requested capabilities all match the actual capabilities, the match is successful - return true.
     */
    public static boolean capabilitiesMatch(List requestedCapabilities, List actualCapabilities)
    {
        //Enumerate over requested capabilities to see if all requested capabilites are contained in the actual List
        // of capabilities.
        Iterator requestedIt = requestedCapabilities.iterator();
        ArrayList matchingCapabilities = new ArrayList();
        while ( requestedIt.hasNext() ) {
            Object requestedCapabilityO = requestedIt.next();
            if ( ! (requestedCapabilityO instanceof AgentCapability ) ) return false;
            AgentCapability requestedCapability = (AgentCapability) requestedCapabilityO;
            // Compare each requested capability with a actual capability.
            Iterator actualIt = actualCapabilities.iterator();
            while ( actualIt.hasNext() ) {
                Object actualCapabilityO = actualIt.next();
                if ( ! (actualCapabilityO instanceof AgentCapability) ) return false;
                AgentCapability actualCapability = (AgentCapability) actualCapabilityO;
                if ( requestedCapability.getName().equals(actualCapability.getName()) ) {
                    if ( requestedCapability.containedInCapability(actualCapability)) {
                        matchingCapabilities.add(requestedCapability); 
                        break;
                    }
                }
            }// end while ( actualIt.hasMoreElements() )
        }// end while  ( requestedIt.hasMoreElements() ) 
        if ( matchingCapabilities.size() == requestedCapabilities.size() ) return true;
        return false;
    
    }
    
    /**
     * Instantiates AgentCapability. 
     */
    public AgentCapabilityImpl()
    {
    }
    
    /**
     * Instantiates AgentCapability. 
     */
    public AgentCapabilityImpl(String name)
    {
        setName(name);
    }
    
    /** 
     * Set the name of this AgentCapability.
     *
     * @param name      String containing the name of the AgentCapability. 
     */
    public void setName(String name)
    {
        set(NAME, name);   
    }
   
    /**
     * Obtain the AgentCapability name.
     *
     * @return String containing the name of the AgentCapability. 
     */
    public String  getName()
    {
        return (String)get(NAME); 
    }

    /** 
     * Set the keywords used to search for this capability.
     *
     * @param keywords  List of Strings containing keywords to be used in search for this capability. 
     */
    public void setKeywords(List keywords)
    {
        if ( keywords == null ) return;
        // new ArrayList to hold a copy of the provided list
    	// otherwise, the caller's List would be modified by this method
    	ArrayList aCopy = new ArrayList(keywords.size());
		Iterator keyIt = keywords.iterator();
		while(keyIt.hasNext())
		{
			Object keyObject = keyIt.next();
			if ( keyObject instanceof String )
			{
				aCopy.add((String) keyObject);
			}
		}
		set(KEYWORDS, aCopy);
    }
   
    /**
     * Obtain a List of String describing keywords to be used in search for this capability.
     *
     * @return List     containing Strings to describe this capability. 
     */
    public List getKeywords()
    {
        if ( ! containsKey( KEYWORDS ) )
            set(KEYWORDS, new ArrayList());
        
        List wordList =  (List)get(KEYWORDS);
        ArrayList aCopy = new ArrayList(wordList.size());
		Iterator it = wordList.iterator();
		while(it.hasNext())
		{
	        aCopy.add((String)it.next() );
		}
		return aCopy;
    }
    
    /** 
     * Set the capability cost
     *
     * @param cost      double containing the cost of this capability.
     */
    public void setCost(double  cost)
    {
        set(COST, new Double(cost));   
    }
   
    /**
     * Obtain the capability cost.
     *
     * @return double   containing the cost of this capability.
     */
    public double getCost()
    {
        Double dCost = (Double)get(COST);
        return dCost.doubleValue();
    }
    
    /** 
     * Set a List of Strings describing how an agent requests another agent to perform the capability. 
     *
     * @param requestForInfo   List of String describing how to request this capabiity. 
     */
    public void setRequestForInfo(List requestForInfo)
    {
        if ( requestForInfo == null ) return;
        // new ArrayList to hold a copy of the provided list
    	// otherwise, the caller's List would be modified by this method
    	ArrayList aCopy = new ArrayList(requestForInfo.size());
		Iterator reqIt = requestForInfo.iterator();
		while(reqIt.hasNext())
		{
			Object reqObject = reqIt.next();
			if (reqObject instanceof String)
			{
				aCopy.add((String)reqObject);
			}
		}
		set(WORDS_TO_REQUEST_CAPABILITY, aCopy);
    }
    
    /** 
     * Return Strings describing how to request this capabiity from an agent. 
     *
     * @return  List    containing String to request this capability from an agent.
     *               
     */
    public List getRequestForInfo()
    {
       if ( ! containsKey( WORDS_TO_REQUEST_CAPABILITY ) )
            set(WORDS_TO_REQUEST_CAPABILITY, new ArrayList());
        
        List infoList =  (List)get(WORDS_TO_REQUEST_CAPABILITY);
        ArrayList aCopy = new ArrayList(infoList.size());
		Iterator it = infoList.iterator();
		while(it.hasNext())
		{
	        aCopy.add((String)it.next() );
		}
		return aCopy;
    }
   
    /** 
     * Set the reliability of the capability (double/percentage).
     *
     * @param reliability   double specifying the reliability of the capability.
     *               
     */
    public void setReliability(double  reliability)
    {
        set(RELIABILITY, new Double(reliability));  
    }
    
    /** 
     * Obtain a double specifying the reliability of the capability (double/percentage). 
     *
     * @return double   conatining the the reliability of the capability.
     *               
     */
    public double getReliability()
    {
        Double dRel = (Double)get(RELIABILITY);
        return dRel.doubleValue();
    }

    /** 
     * Determine if this AgentCapability is contained in the other AgentCapability,
     * meaning that all properties of this AgentCapability are present in the other.
     *
     * @param otherCapability     AgentCapability to compare this AgentCapability with.
     *
     * @return boolean   indicating if the properties of this AgentCapabilities
     *                   are present in the other AgentCapability.
     *               
     */
    public boolean containedInCapability(AgentCapability otherCapability)
    {
        Enumeration myPropertiesEnum = keys();
        ArrayList matchingProps = new ArrayList();
        int myPropsSize = 0;
        
        while ( myPropertiesEnum.hasMoreElements() ) {
            myPropsSize ++;
            String myPropertyName = (String) myPropertiesEnum.nextElement();
            
            if (! (otherCapability instanceof AgentCapabilityImpl) ) return false;
            AgentCapabilityImpl other = (AgentCapabilityImpl) otherCapability;
            if ( other.containsKey(myPropertyName) ){
                Object myPropertyValue = get(myPropertyName);
                Object otherValue = other.get(myPropertyName);                
                if ( myPropertyValue.equals(otherValue)) {
                    matchingProps.add(myPropertyName);  
                }
            }
        }// end while
        // If all search params were either partially contained or fully contained in the found KAoSAgentDescription,
        // then add the KAoSAgentDescription to the result.
        if ( matchingProps.size() == myPropsSize ) { 
            return true;
        }
        return false;
    }
	
	public String toString()
	{
		return getName();	
	}
    
}
