/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2005/11/28 21:59:11 $
 */
package kaos.core.csi;

//import kaos.core.csi.registration.Registration;
import kaos.core.csi.registration.Property;
import kaos.core.csi.registration.PropertyImpl;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.core.util.Logger;

import javax.agent.service.transport.NotLocatableException;
import javax.agent.service.transport.TransportFailure;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import java.io.Serializable;


/**
 * $Revision: 1.5 $
 */
public class BasicKAoSActorImpl
    implements  KAoSActor
   {

    //KAoSActor vars
    protected KAoSAgentDescription      _actorDesc; 
    protected Logger                    _logger = Logger.getLogger(this);
    protected Hashtable                 _transports;
    protected boolean                   _registered;
    /**
     * Instantiates a description of a KAoSActor. 
     */
    public BasicKAoSActorImpl(KAoSAgentDescription desc)
    {
        _actorDesc = desc;
    }

    //////////////////////////////////////////////////////////////
    // Implementation of the KAoSActor interface
   
    /** 
     * Get the name of the concept mapping between the entity system's ontological concepts and KAoS ontology. 
     *
     * @return       String specifying the name of the mapping to be used to translate ontological names from
     *               the entity system to KAoS ontologies.  The default name is "KAoS" indicating KAoS ontologies
     *               to be used, no translation necessary.
     */
    public String getConceptMappingName()
    {
        return _actorDesc.getConceptMappingName();
    }
   
    /** 
     * Returns a List of String containing the names of domains the actor is registered in.  
     * Note: When the actor first registers with the DS, this List is checked and, if not empty, the actor 
     * will be registered in the specified domains.  Subsequently, if the actor wants to register 
     * in additional domains, then the actor has to lookup the desired domain and register in it 
     * via the Domain.registerAgent() method, which will update this List after successful registration.
     *
     * @return       List of domain names contained by this KAoSActor.
     */
    public List getDomainNames()
    {
        return _actorDesc.getDomainNames();
    }

    /**
     * Obtain the actor's globally unique identifier.
     *
     * @return String representing the actor's guid.
     */
    public String getGUID()
    {
        return _actorDesc.getEntityNameAsString();
    }
    
    /**
     * Obtain the actor's name.
     *
     * @return String representing the actor's name.
     */
    public String getName()
    {
        return _actorDesc.getAgentNickname();
    }
    
    /**
     * Obtain the collection of ontological types of this KAoSActor. 
     *
     * Presently, the types are defined by constants in kaos.ontology.vocabulary.ActorConcepts.  
     *
     * @return Map containing mappings of ontological type names -> OWL descriptions of the types 
     *         or null, if ontological types have not been set.
     */
    public Map getEntityOntologicalTypes()
    {
        return _actorDesc.getEntityOntologicalTypes();
    }
    
    /**
     * Obtain the collection of just names of ontological types of KAoS entity described by this KAoSActor. 
     *
     * Presently, the types are defined by constants in kaos.ontology.jena.ActorConcepts.  
     *
     * @return Enumeration containing names of ontological types or null, if there no types specified in the KAoSActor.
     */
    public Enumeration getEntityOntologicalTypeNames()
    {
        return _actorDesc.getEntityOntologicalTypeNames();
        
    }
    
    /**
     * Obtain guarded/unguarded status of this KAoSActor.
     *
     * @return boolean indicating whether the KAoSActor is guarded. 
     */
    public boolean isGuarded()
    {
        List guardIDs = _actorDesc.getGuardIDs();
        return ( guardIDs.size() > 0 );
    }
    
     /**
     * Set the  registered/unregistered status of this KAoSActor.
     *
     * @param  registered boolean indicating whether the KAoSActor has been registered in the KAoS DS. 
     */
    public void setRegistered(boolean registered)
    {
        if ( ! registered ) return; //this is always registered
        _registered = registered;
        logMessage(".setRegistered: " + _actorDesc.getEntityNameAsString() + " registration status has been set to: "
                   + registered + ".", Logger.INFO);
    }
    /**
     * Obtain registered/unregistered status of this KAoSActor.
     *
     * @return boolean indicating whether the KAoSActor has been registered in the KAoS DS. 
     */
    public boolean isRegistered()
    {
        return _registered;
    }
    
    /**
     * Obtain the collection of AgentCapabilities of this KAoSActor. 
     *
     * @return List of AgentCapability describing the capabilities  of the actor.
     */
    public List getCapabilities()
    {
        return _actorDesc.getCapabilities();
    }
    
    ////    ******  GENERIC PROPERTY MANAGMENT API 
        
    /**
     * Returns a Hashtable containing a mapping of actor properties. 
     * The key is the Property name, the value is the Property.
     * These properties are stored in the Directory Service.
     *
     * @return Hashtable of actor's properties.  
     */
    public Hashtable getProperties ()
    {
        Hashtable properties = new Hashtable();
        Enumeration myEnum = _actorDesc.keys();
        while ( myEnum.hasMoreElements() ) {
            String key = (String)myEnum.nextElement();
            Object value = _actorDesc.get(key);
            if ( (value instanceof Serializable) ) {
                Property prop = new PropertyImpl();
                prop.setName(key);
                prop.setValue((Serializable)value);
                properties.put(key,prop);
            }
        }
        return properties;
    }
        
    /**
     * Return the property, whose name is specified
     *
     * @param name  String containing the name of the property to be returned.
     */
    public Property getProperty(String name)
    {
        Object value = _actorDesc.get(name);
        if ( (value instanceof Serializable) ) {
            Property prop = new PropertyImpl();
            prop.setName(name);
            prop.setValue((Serializable)value);
            return prop;
        }
        return null;
    }
    
    /**
     * Return the value of the given property
     *
     * @param property  Property, whose value is to be returned.
     *
     * @return Object   containing the given Property value.
     */
    public Object getPropertyValue(Property property)
    {
        return _actorDesc.get(property.getName());
    }
    
    ////    ******  TRANSPORT API 
    /**
     * Send  the specified  message content to the given KAoSActor.
     *
     * @param messageContent        Serializable containing the message to be sent out.
     *
     * @param receiver          KAoSAgentDescription containing the description of the receiver.
     *
     * @throws  NotLocatableException when the receiver cannot be located.
     *
     * @throws  TransportFailure when the transport system fails.
     */
    public void sendMessage(Serializable messageContent, KAoSActor receiver) 
        throws NotLocatableException,
               TransportFailure
    {
        throw new TransportFailure("    *** Transport not available!! Unable to send a message: "
                                   +  messageContent + " to: " + receiver.getName());
    }
               
    /**
     * Receive  the specified  message content from the given KAoSActor.
     *
     * @param messageContent        Serializable containing the received message. 
     *  
     * @param sender                KAoSAgentDescription containing the description of the sender. 
     */
    public void receiveMessage(Serializable messageContent,  KAoSActor sender)
    {
        System.out.println("    Unable to receive and process message: " + messageContent 
                           + " from sender: " + sender.getName());
    }
    
    
    /**
     * Return the List of valid Transports.
     *
     * @return List     containing the supported Transports.
     */
    public List getTransports()
    {
        ArrayList tranList = new ArrayList();
        Enumeration myEnum = _transports.elements();
        while ( myEnum.hasMoreElements() ) {
            Transport transport = (Transport)myEnum.nextElement();
            tranList.add(transport);
        }
        
        return tranList;
    }
    
    /**
     * Return the Transport, whose name is specified.
     * 
     * @param name          String containing the Transport name to be retrieved.
     *
     * @return Transport    whose name was specified.
     */
    public Transport getTransport(String name)
    {
        return (Transport) _transports.get(name);
    }

    //////////////////////////////////////////////////////////////
    public KAoSAgentDescription getAgentDescription()
    {
        return _actorDesc;
    }

    protected void logMessage(String msg, int logLevel)
    {
   		if (_logger.isEnabledFor(logLevel))
   			_logger.logMessage(msg, logLevel);
    }
	
    protected void logMessage(String msg, Exception e, int logLevel)
    {
   		if (_logger.isEnabledFor(logLevel))
   			_logger.logMessage(msg, e, logLevel);
    }
   //////////////////////////////////////////////////////////////
   
}

