package kaos.core.csi.observe;

import kaos.core.csi.CSIFactory;
import kaos.core.csi.query.QueryRegistration;
import kaos.core.csi.transport.TransportSupport;
import kaos.core.service.directory.KAoSAgentDescription; 
import kaos.core.service.directory.KAoSEntityDescription;
import kaos.core.util.ExecEnvUtility;
import kaos.core.util.MsgUtils;
import kaos.core.util.Logger;
import kaos.core.util.Msg;
import kaos.core.util.MethodCallRequestMsg;
import kaos.core.util.MethodCallResultMsg;

import javax.agent.service.transport.*;
import javax.agent.Locator;
import javax.agent.TransportMessage;

import ri.acr.FipaAclPerformative;

import java.lang.reflect.*;
import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;

/**
 * This class implements the Command interface, which defines sending 
 * commands to entities
 */
public class ObserverManagerImpl
    implements ObserverManager, MessageListener
               
{
    
    private static final ObserverManagerImpl    __INSTANCE = new ObserverManagerImpl() ;
    private static boolean                      __INITIALIZED = false;
    private static final String                 _NICKNAME_ROOT = "CSI-ObserverManager-";
    private static final String                 _REGISTER_OBSERVER = "register-observer";
    private static final String                 _DEREGISTER_OBSERVER = "deregister-observer";
    
    protected Locator                   _myLocator;
    protected String                    _nickname;
    protected QueryRegistration         _query;
    protected TransportSupport          _transport;
    protected String                    _transportName;	
    protected Hashtable                 _observers = new Hashtable();
    protected Hashtable                 _observablesDescriptions = new Hashtable();
    
	private Logger		                _logger = Logger.getLogger(this);
    
    
    /**
     * Retrieve an instance of ObserverManager.
     *     
     * @return a singleton, ObserverManager.
     */
    public static ObserverManager getInstance(String transportName)
        throws Exception
    {
        if ( __INITIALIZED ) return __INSTANCE;
        return getInstance(transportName, null);
    }
    
    /**
     * Retrieve a reference to the ObserverManager.
     *
	 * @param       transportName   is a String containing  the name of the transport to use by the ObserverManager.
	 *                                      
	 * @param       nickName        is a String containing  the nick name of the ObserverManager that may be used by certain
	 *                              transport implementations, or null if not required by the transport.
     *
     * @return a reference to the ObserverManager
     */
    
    public static ObserverManager getInstance(String transportName, String nickName)
        throws Exception
    {
        if ( __INITIALIZED ) return __INSTANCE;
            TransportSupport transport = CSIFactory.getTransportSupport();
            if ( (transportName == null) || ( ! transport.isTransportSupported(transportName) )  )
                throw new Exception("ObserverManager: transport name is invalid: " + transportName 
                                    + "! Please specify valid transport! Valid transport names: "
                                    + transport.getSupportedTransportsNames());
            if ( nickName == null ) nickName = _NICKNAME_ROOT + ExecEnvUtility.getVMID();
            __INSTANCE.initialize(transportName, nickName);
            __INITIALIZED = true;
            return __INSTANCE;
    }
    
    private ObserverManagerImpl()
    {
    }

    /*****************************************************************************************
       kaos.core.csi.observe.ObserverManager interface implementation - begin
    *****************************************************************************************/
    
    /** 
     * Register the given KAoSObserver with the KAoSObservable, whose GUID is given for updates
     * described by the updateType and updateConditions.
     *
     * @param observer           KAoSObserver requesting registration for updates. 
     *
     * @param updateType         String indicating type of update the KAoSObserver is interested in.
     *
     * @param updateCondition    Serializable describing the conditions under which to send the update.
     *
     * @param observableGUID     String containing the GUID of the KAoSObservable to register with.
     *
     * @throws	    NotLocatableException   when the KAoSObservable cannot be located.
	 *				                        
	 * @throws	    TransportFailure        when transport services fails.
     *
     */
    public void registerObserver(KAoSObserver observer,
                                String updateType, 
                                Serializable updateCondition,
                                String observableGUID)
        throws NotLocatableException, TransportFailure
    {
        printDebugString("registerObserver: received updateType " + updateType + "\n   updateCondition: " 
		                  + updateCondition + "\n   register with: " + observableGUID, Logger.INFO);
		                  
		validateObserver(observer);
				
		// Get observable description		
        KAoSAgentDescription observableDesc = getAgentDescription(observableGUID);
        if ( observableDesc == null ) throw new NotLocatableException("Unable to locate observable: " + observableGUID);
		
        registerObserver(observer,
						 updateType,
                         updateCondition,
                         observableDesc);
                                
    }
    
   /** 
    * Register the given KAoSObserver with the KAoSObservable, whose KAoSEntityDescription is given, for updates
    * described by the updateType and updateConstraints.
    *
    * @param observer           KAoSObserver registering with for updates. 
    *
    * @param updateType         String indicating type of update the KAoSObserver is interested in.
    *
    * @param updateCondition    Object describing the conditions under which to send the update. 
    *
    * @throws	    NotLocatableException   when the KAoSObservable cannot be located.
	*				                        
	* @throws	    TransportFailure        when transport services fail.
    
    */
   public void registerObserver(KAoSObserver observer,
                                String updateType, 
                                Serializable updateCondition,
                                KAoSEntityDescription observableDesc)
    throws NotLocatableException, TransportFailure
    {
        printDebugString("registerObserver: received updateType " + updateType + "\n   updateCondition: " 
		                  + updateCondition + "\n   register with: " + observableDesc, Logger.INFO);
		Vector args = new Vector(3);
	    args.addElement(updateType);
        if (updateCondition == null) {
            updateCondition = new Hashtable();
        }            
        args.addElement(updateCondition);
	    
		validateObserver(observer);

        // we need to add the observer to the hashtable before we send the registration message
        // to avoid a race condition where the initial updates are received before the observer 
        // is in the hashtable
        _observers.put(observer.getID(), observer);

        try {
            MyRemoteKAoSObserver remoteObserver = new MyRemoteKAoSObserver(_myLocator,
                                                                               _nickname,
                                                                               observer.getID());
            args.addElement(remoteObserver);															       
            sendMsg(observableDesc,
                    "registerObserver",
                    args,
                    remoteObserver);
        }
        catch (NotLocatableException xcp) {
            // todo: we need to store the observers in a more complicated structure than
            // a hashtable, because it may be that an observer registers for multiple updateTypes
            // and updateConditions, and we should only remove the observer from the
            // structure if this is the only updateType and updateCondition that it
            // tried to register for
            // But this won't hurt anything for now, it just keeps an extra entry
            // in the table
            throw xcp;
        }
        catch (TransportFailure xcp) {
            throw xcp;                        
        }
    }
    
    /** 
    * Register the given KAoSObserver with the KAoSObservable, whose GUID is given for updates
    * described by the updateType and updateConditions.
    *
    * @param observer           KAoSObserver requesting registration for updates. 
    *
    * @param updateType         String indicating type of update the KAoSObserver is interested in.
    *
    * @param updateCondition    Serializable describing the conditions under which to send the update.
    *
    * @param observableGUID     String containing the GUID of the KAoSObservable to register with.
    
    * @param brokerGUID         String containing the GUID of the broker, through which
    *
    * @throws	    NotLocatableException   when the KAoSObservable or broker cannot be located.
	*				                        
	* @throws	    TransportFailure        when transport services fail.
                               the rgistration will be processed.
    */
   public void registerObserver(KAoSObserver observer,
                                String updateType, 
                                Serializable updateCondition,
                                String observableGUID,
                                String brokerGUID)
        throws NotLocatableException, TransportFailure
    {
        System.out.println("registerObserver: NOT IMPLEMENTED YET!");
    }
                                
   /** 
    * Deregister the given KAoSObserver from the registry of observers asoociated
    * with the KAoSObservable, whose GUID is given.
    *
    * @param observer           KAoSObserver deregistering. 
    *
    * @param updateType         String indicating type of update the KAoSObserver is no longer interested in.
    *
    * @param updateCondition    Serializable containing the condition the KAoSObserver is no longer interested in.
    *
    * @param observableGUID     String specifying the GUID of the observable, from which to deregister 
    *                           for the given update type.
    *
    * @throws	    NotLocatableException   when the KAoSObservable cannot be located.
	*				                        
	* @throws	    TransportFailure        when transport services fail.
    */
    public void deregisterObserver(KAoSObserver observer,
                                   String updateType,
								   Serializable updateCondition,
                                   String observableGUID)
        throws NotLocatableException, TransportFailure  
    {
        printDebugString("deregisterObserver: received request to deregister observer for update type: " 
                         + updateType + "\n From observable: " + observableGUID, Logger.INFO);
		                  
		validateObserver(observer);
				
		// Get observable description		
        KAoSAgentDescription observableDesc = getAgentDescription(observableGUID);
        if ( observableDesc == null ) throw new NotLocatableException("Unable to locate observable: " + observableGUID);
		
        deregisterObserver(observer,
                           updateType,
						   updateCondition,
                           observableDesc);
    }
   
    /** 
     * Deregister the given KAoSObserver from the registry of observers asoociated
     * with the KAoSObservable, whose GUID is given.
     *
     * @param observer           KAoSObserver deregistering. 
     *
     * @param updateType         String indicating type of update the KAoSObserver is no longer interested in.
     *
     * @param updateCondition    Serializable containing the condition the KAoSObserver is no longer interested in.
     *
     * @param observableDesc     KAoSEntityDescription of the observable, from which to deregister 
     *                           for the given update type.
     *
     * @throws	    NotLocatableException   when the KAoSObservable cannot be located.
	 *				                        
	 * @throws	    TransportFailure        when transport services fail.
     */
    public void deregisterObserver(KAoSObserver observer,
                                   String updateType,
								   Serializable updateCondition,
                                   KAoSEntityDescription observableDesc)
        throws NotLocatableException, TransportFailure
    {
        printDebugString("deregisterObserver: received request to deregister observer for update type: " 
                         + updateType + "\n From observable: " + observableDesc, Logger.INFO);
                         
		validateObserver(observer);
		
		Vector args = new Vector(2);
	    args.addElement(updateType);
        if (updateCondition == null) {
            updateCondition = new Hashtable();
        }
        args.addElement(updateCondition);
	    
	    MyRemoteKAoSObserver remoteObserver = new MyRemoteKAoSObserver(_myLocator,
																	       _nickname,
																	       observer.getID());
		args.addElement(remoteObserver);															       
	   sendMsg(observableDesc,
	           "deregisterObserver",
	           args,
			   remoteObserver);
						    
        _observers.remove(observer.getID());
    }
   
	/*****************************************************************************************
       kaos.core.csi.observe.ObserverManager interface implementation - end
    *****************************************************************************************/
	/*****************************************************************************************
       MessageListener interface implementation - begin
    *****************************************************************************************/
    /**
     * Implements the receiveMessage method mandated by the
     * MessageListener interface.  This method prints
     * the incoming TransportMessage to System.out.
     *
     * @see MessageListener
     * @see TransportMessage
     * @see "JAS Specification, Section(s) 3.4.2, 4.7.2"
     */
    public void receiveMessage( TransportMessage tmsg )
    {
		printDebugString("receiveMessage: Incoming transport message:\n"+ tmsg, Logger.DEBUG);
        
        Object msgObject= MsgUtils.unwrapObjectContent(tmsg);
		printDebugString("receiveMessage: Object content of the message:\n"+ msgObject + "\n", Logger.DEBUG);

        Msg msgContent = MsgUtils.unwrapMsgContent(tmsg);
		
		if ( (msgContent instanceof MethodCallRequestMsg) ) {
		    satisfyRequest((MethodCallRequestMsg) msgContent);
	    }
		else {
            printDebugString(".receiveMessage: received message is of unknown type, not processing: " +
							 msgContent + "\n type = " + msgContent.getClass().getName(), Logger.DEBUG);
            return;
        }
    }

    /**
     * Two agents are equal if their respective hash codes
     * are equal.
     *
     * @return true if obj equals this AgentUsecase.
     *
     * @see #hashCode
     * @see "JAS Specification, Section(s) 4.7.2"
     */
    public boolean equals( Object obj )
    {
        if ( obj == this )
            return true;

        if ( !(obj instanceof ObserverManagerImpl) )
            return false;

        return this.hashCode() == ((ObserverManagerImpl)obj).hashCode();
    }

    /**
     * Determines the hash code for the AgentUsecase.  The AgentUsecase
     * hash code is simply the hash code of the represented agents
     * AgentName.  If the AgentName field has not yet been Set,
     * then the hash code is found from <code>super.hashCode()
     * </code>.
     *
     * @return int the hashCode for this AgentUsecase.
     *
     * @see Object#hashCode
     * @see "JAS Specification, Section(s) 4.7.2"
     */
    public int hashCode()
    {
        return _nickname == null ? super.hashCode() : _nickname.hashCode();
    }
	/*****************************************************************************************
       MessageListener interface implementation - end
    *****************************************************************************************/

    /////////////////////////////   PUBLIC METHODS BEGIN 
    /** 
     * Receive an update from MyRemoteKAoSObserver
     *
     * @param observerID         The ID of the observer to inform
     * @param observableID       String identifying the KAoSObservable sending the update. 
     * @param updateType         String indicating type of update.
     * @param updateCondition    Object indicating the update condition.
     * @param update             Object containing the update. 
     *
     */
    public void updateObserver (String observerID,
                                String observableID,
                                String updateType, 
                                Object updateCondition,
                                Object update)
    {
		printDebugString(".update: received update of:" +
						 "\n\t type: " + updateType +
						 "\n\t condition: " + updateCondition +
						 "\n\t value: " + update +
						 "\n\t from: " + observableID, Logger.INFO);
                         
        // since Hashtable is synchronized, we don't need to worry about putting this is a sync
        // block unless we get an iterator from it
        if (!_observers.containsKey(observerID)) {
            _logger.logMessage("update: no observer registered for ID " + observerID, Logger.WARN);
            return;
        }
        
	   //HACK because ObserverManager cannot send null
  	    if ( updateCondition instanceof Hashtable) {
			if( ((Hashtable)updateCondition).size() == 0){
				updateCondition = null;
			}
	    }	        
        
        KAoSObserver observer = (KAoSObserver) _observers.get(observerID);
        try {
            observer.update(observableID, updateType, updateCondition, update);
			printDebugString(".update: updated:" +
						 "\n\t id: " + observableID +
						 "\n\t type: " + updateType +
						 "\n\t condition: " + updateCondition +
						 "\n\t value: " + update, Logger.INFO);
			
        }
        catch (NotificationException xcp) {
            _logger.logMessage("Observer threw notification exception" + xcp, Logger.WARN);
        }
    }
 
    public String getID()
    {
        return _nickname;
    }
    
    /////////////////////////////   PUBLIC METHODS END
    
    /////////////////////////////   PROTECTED METHODS BEGIN
     protected MethodCallResultMsg satisfyRequest (MethodCallRequestMsg requestMsg)
    {
        MethodCallResultMsg resultMsg = null;
        try {
            
            String methodName = requestMsg.getMethodName();
            Vector args = requestMsg.getArgs();
            if ( args != null ) {
                Object argObjects[] = new Object[args.size()];
                for (int i=0; i<args.size(); i++) {
                    argObjects[i] = args.elementAt(i);
                }

                // we have to check this the "hard way" because Class.getMethod(name, args[])
                // does not work if the args are subclasses of the parameter classes

                // find the method matching the method name requested
                Method methods[] = this.getClass().getMethods();
                boolean foundMethod = false;
                for (int i=0; i<methods.length; i++) {
                    if ( methods[i].getName().equals(methodName) ) {
                        // check to see if the parameter classes match the arguments
                        Class parameters[] = methods[i].getParameterTypes();
                        boolean classesMatch = false;
                        if ( parameters.length == argObjects.length) {
                            classesMatch = true ;
                            for (int j=0; j<parameters.length; j++) {
                                if (!parameters[j].isInstance(argObjects[j])) {
                                    classesMatch = false;
                                    break;
                                }
                            }
                        }
                        // if we found the right method, execute it
                        if ( classesMatch ) {
                            foundMethod = true;
                            Object result = methods[i].invoke(this, argObjects);
                            resultMsg = new MethodCallResultMsg(requestMsg.getSequenceId(),
                                                                result);
                            break;
                        }
                    }
                }
                // if we didn't find the method, throw an exception
                if ( ! foundMethod ) {                    
                    throw new NoSuchMethodException(methodName + " args: " + args);
                }
            }
            // args == null, invoke method with no args
            else {
                Object result = this.getClass().getMethod(methodName, null).invoke(this, null);
                resultMsg = new MethodCallResultMsg(requestMsg.getSequenceId(),
                                                    result);
            }
        }
        catch (Exception xcp) {
            if ( xcp instanceof java.lang.reflect.InvocationTargetException ) {
                java.lang.reflect.InvocationTargetException inex = (java.lang.reflect.InvocationTargetException) xcp;
                resultMsg = new MethodCallResultMsg(requestMsg.getSequenceId(),
                                                inex.getTargetException());
            }
            else {
                resultMsg = new MethodCallResultMsg(requestMsg.getSequenceId(),
                                                xcp);
            }
            if (_logger.isEnabledFor(Logger.ERROR))
                _logger.logMessage("", xcp, Logger.ERROR);
                
        }

        return resultMsg;
    }
    
    
    /////////////////////////////   PRIVATE METHODS BEGIN
    private void initialize(String transportName, String nickName)
        throws Exception
    {
        printDebugString("initialize: received transport name: " + transportName
		                 + " nickName: " + nickName, Logger.INFO);
		_transport = CSIFactory.getTransportSupport();
        _nickname =  nickName;
        _transportName = transportName;        
        _myLocator = _transport.createLocator(_nickname, null, _transportName);
        _transport = CSIFactory.getTransportSupport();
         
        try {
            _transport.bindMsgListenerToTransport(this,
                                                 _nickname,
                                                 _myLocator,
                                                 _transportName);
			printDebugString("initialize: successfully bound my locator to transport. Locator:\n " + _myLocator, Logger.INFO);
        }
        catch(Exception e) {
            e.printStackTrace();
            throw e;
        }		
	}    
    
    /**
    */
    private KAoSAgentDescription getAgentDescription(String agentID)
    {
        if ( _observablesDescriptions.containsKey(agentID) ) {
            printDebugString(".getAgentDescription: returning stored description for agentID: "  + agentID, Logger.INFO);
            return (KAoSAgentDescription)_observablesDescriptions.get(agentID);
        }
            
        if ( _query == null ) {
            try {
                initQuery();
            }
            catch(Exception e) {
                e.printStackTrace();
                return null;
            }
        }
        
        KAoSAgentDescription desc = null;
        printDebugString(".getAgentDescription: will request description for agentID: "  + agentID, Logger.INFO);
		if ( agentID == null ) return null;
        try {
			desc = _query.getActor(agentID);
        }
        catch(Exception e) {
            e.printStackTrace();
            return null; 
        }
        
        if ( desc != null ) _observablesDescriptions.put(agentID, desc);
        printDebugString(".getAgentDescription: obtained description from DS: \n"  + desc, Logger.INFO);
        return desc;
    }
    
    /**
     */
    private void initQuery()
        throws Exception
    {
        CSIFactory.setCurrentTransport(_transportName);
        _query = CSIFactory.getQueryRegistration();
    }
    
    /**
     */
    private void validateReceiverDescription(KAoSEntityDescription receiverDesc)
        throws NotLocatableException
    {
        assertNotLocatable( receiverDesc == null, "\n   Receiver's description is null!  Cannot send a request!");
        
        Locator[] receiverLocs = receiverDesc.getLocators();
        assertNotLocatable( receiverLocs.length == 0, "\n   Receiver's Locators are empty!  Description:\n " + receiverDesc);
        
        String receiverName = receiverDesc.getEntityNameAsString();
        assertNotLocatable( receiverName == null,  "\n   Receiver's name is null!  Description:\n " + receiverDesc);
        
    }
    
    /**
     */
    private void validateObserver(KAoSObserver observer)
        throws IllegalArgumentException
    {
        if ( observer == null)
            throw new IllegalArgumentException("\n   KAoSObserver is null!  Will not proceed!");
    }
    
    /**
     * @throws NotLocatableException if exp is true.
     */
    public void assertNotLocatable( boolean exp, String msg )
        throws NotLocatableException
    {
        if ( exp )
         throw new NotLocatableException( "Assert not locatable:\n " + msg );
    }	
		
	/**
	    Use MethodCallRequest to send the specified request for the given action
	    to the action executor, whose KAoSEntityDescription is given.
	    	    
	    Return the unique ID identifying the sent request.
	 */
	private String sendMsg(KAoSEntityDescription observableDesc,
						   String methodName,
						   Vector args,
						   RemoteKAoSObserver remoteObserver)
		throws  NotLocatableException, TransportFailure
    {
        printDebugString("sendMsg: received observableDesc: \n" + observableDesc + "\n   update type: " 
		                  + methodName + "\n   update args: \n" + args, Logger.INFO); 
        validateReceiverDescription(observableDesc);
	    
        if  ( methodName == null ) {
            printDebugString("sendMsg: requested method name is null! \n ", Logger.INFO);
            throw new IllegalArgumentException();
        }   
		
		String receiverName = observableDesc.getEntityNameAsString();
        Locator[] receiverLocs = observableDesc.getLocators();
        
        // Create MethodCallRequestMsg 
        MethodCallRequestMsg request = new MethodCallRequestMsg();
        String requestID = request.getSequenceId();
        		
	    //set method o request action and add args
	    request.setMethodNameArgs(methodName, args);
		
	    printDebugString("sendMsg: will try to send the following request: \n  MethodCallRequestMsg: "
                   + request + "\n     observableDesc: "   + observableDesc, Logger.INFO);
         
       // The sendMessageTo() operation may throw 2 exceptions:
       // TransportFailure and NotLocatableException
	    _transport.sendMessageTo(_myLocator,
							     receiverLocs[0],
							     _nickname,
								 receiverName,
                                 FipaAclPerformative.REQUEST,
                                 request,
							     _transportName); 
							     
		printDebugString("sendMsg: message sent!", Logger.INFO);
		return requestID;
    }   
    
    //////////////////////////// PRIVATE METHODS END
	
    // Utility methods ///////////////////////////////////////////
      
	protected void printDebugString (String msg, int loggerDisplayLevel)
    {
        if (_logger.isEnabledFor(loggerDisplayLevel))
   	        _logger.logMessage(msg, loggerDisplayLevel);
    }

	///////////////////////////////////////////////////////////////////////////////
    public static void main(String [] args)
      throws Exception      
    {
        String transport = null;
        String name = null;
        if (args.length > 0) {
	        int i = 0;
	        while (i < args.length) {
	            if (args[i].equalsIgnoreCase ("-name")) {
	                if ((i+1) < args.length) {
    	                name = args[i+1];
    	                i++;
    	            }
    	            else {
    	                System.out.println ("Must specify name with the -name keyword.");
    	                System.exit(0);
    	            }
	            }
	            else if (args[i].equalsIgnoreCase ("-transport")) {
	                if ((i+1) < args.length) {
    	                transport = args[i+1];
    	                i++;
    	            }
    	            else {
    	                System.out.println ("Must specify transport with the -transport keyword.  Valid transpors: corba, grid, tcp.");
    	                System.exit(0);
    	            }
	            }
	            i++;
	        }
	    }

        System.out.println ("\n     Requested name:      " + name);
        System.out.println ("\n     Requested transport: " + transport);
        
        ObserverManager observe = null;
        if ( (name == null) ){
            observe = ObserverManagerImpl.getInstance(transport);
        }
        else {
            observe = ObserverManagerImpl.getInstance(transport, name);
        }
        
        System.out.println ("\n Got observe instance" + observe + "\n\n");
		
        synchronized ( Thread.currentThread() ) {
            Thread.currentThread().wait();

        }
   }
    
    public static class MyRemoteKAoSObserver extends RemoteKAoSObserverImpl
    {
        /**
		 * 
		 */
		private static final long serialVersionUID = -7489545469944458868L;

		/*
         * Constructor
         */
        public MyRemoteKAoSObserver(Locator receiverLocator, String receiverName, String observerID)
        {
            super (receiverLocator, receiverName, observerID);
        } 

        /**
         * overrides superclass, to send the update to our method "updateObservers" which
         * includes the ID of the observer to notify. This is needed because we are acting
         * as a proxy for possibly multiple observers with the same updateType and updateCondition,
         * so we need them indexed by their ID so that they don't receive the update more than once
         */
        public void update(String observableID,
                           String updateType, 
                           Object updateCondition,
                           Object update)
            throws NotificationException		
        {
            //System.out.println("\n  RemoteKAoSObserver.update: updateType" + updateType + " update value: " + update);
            if ( (observableID == null)  || (updateType == null) || (update == null)) {
                _logger.logMessage("\n  RemoteKAoSObserver.update: One of the arguments is null: \n updateType" + updateType 
                                   + " update value: " + update + "\n   observable id: " + observableID 
                                   + "\n    Unable to forward update!", Logger.ERROR);
                return;
            }


            if ( ! (update instanceof Serializable ) ) {
                _logger.logMessage("\n RemoteKAoSObserver.update:  received update is not Serializable: " 
                                   +  update + "\n    Unable to foward the update to observer!  " + _observerID, Logger.ERROR);
                return;
            }
             // Create MethodCallRequestMsg 
            MethodCallRequestMsg request = new MethodCallRequestMsg();
            Vector args = new Vector(4);
            args.addElement(_observerID);
            args.addElement(observableID);
            args.addElement(updateType);
            if(updateCondition == null){
                updateCondition = new Hashtable();
            }
            args.addElement(updateCondition);
            args.addElement((Serializable)update);

            //set method o request action and add args
            request.setMethodNameArgs("updateObserver", args);
            try{
                sendMessageObj(request);
            }
            catch(TransportFailure e){
                //logMessage("", e);
                throw new NotificationException("Problem notifying: " + e.getMessage());
            }
            catch(NotLocatableException e2){
                //logMessage("", e2);
                throw new NotificationException("Problem notifying: " + e2.getMessage());
            }		
        }
    }
}

