/**
 * 
 */
package kaos.core.csi.policy;

import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.agent.service.ServiceFailure;

import kaos.core.csi.ActionInstanceDescription;

/**
 * @author KAoS Team
 * 
 * This interface defines operations to advice the user as to which
 * property values are allowed/forbidden for the given action.
 *
 */
public interface PolicyAdvice {
	
	/**
     * Get policies for the specified action type.
     *
     * @param  actionType           String specifying the action type for the requested policies.  
     * 
     * @param pObserver        		PolicyDecisionObserver, an entity interested in receiving updates
	 *  							whenever policy decisions change.
	 *  
     * @return List                 Contains policies defined for the specified action.
     * 
     * @throws ServiceFailure 		when the policy service cannot be reached, or fails.
     *                        
     */
    public List getPoliciesForActionType (String actionType,
    									  PolicyDecisionObserver pObserver) 
    throws ServiceFailure;
		
	/**
     * This function is used when the agent/enforcer has only  partial
     * information about an action and would like to determine what range
     * of properties can be allowed by the policy set.
     *
     * The agent/enforcer partially fills an ActionInstanceDescription object
	 * and sends it to the method, which finds those policies that 
	 * are applicable to this action and contain the given property.
	 * The method will then select only those values for the given properties 
	 * that will not conflict with higher priority policies containing
	 * the given properties.
     *      
     * @param  propertyNames    Vector containing the properties for which
	 *						    values are to be found.
	 *  
	 * @param  actionDesc       ActionInstanceDescription object which will
	 *							be used to find applicable policies.
	 *
	 * @param  tight            boolean, if set to 'true', will result in returning
	 *                          only these values for the missing specified property
	 *                          which would satisfy some policy if used alone.
     *
     * @param pObserver        	PolicyDecisionObserver, an entity interested in receiving updates
	 *  						whenever policy decisions change.
	 *  
	 * @return	Vector			The Vector containing multiple ActionInstanceDescription
	 *							objects which contain the allowed values
	 *							for those properties.
	 *
     * @throws ServiceFailure 	when the policy service cannot be reached, or fails.
     * 
     */
	public Vector getAllowableValuesForActionProperties (Vector propertyNames,
													     ActionInstanceDescription actionDesc,
													     boolean tight,
													     PolicyDecisionObserver pObserver) 
	throws ServiceFailure;
	
	/** 
	 *  In this case, there are NO properties specified. 
	 *  
	 *	The function uses the given, partially filled, ActionInstanceDescription object to 
	 *  find those policies that are applicable to this action, and then it selects all the 
	 *  properties from those policies which are not originally contained in the provided
	 * 	ActionInstanceDescription object. 
	 *  It then chooses only those values for the given properties which do not conflict 
	 *  with their higher priority selves, and returns all those properties in the 
	 *  ActionInstanceDescription object.
	 * 
	 *  @param  actionDesc		ActionInstanceDescription object which will
	 *							be used to find applicable policies.
	 *
     *  @param pObserver        PolicyDecisionObserver, an entity interested in receiving updates
	 *  						whenever policy decisions change.	 
	 *   
	 *	@return	Vector			The Vector containing multiple ActionInstanceDescription
	 *							objects which contain the allowed values
	 *							for those properties.
	 *
	 * @throws ServiceFailure when the policy service cannot be reached, or fails.
	 *
	 */
	public Vector getAllowableValuesForActionProperties (ActionInstanceDescription actionDesc,
														 PolicyDecisionObserver pObserver) 
	throws ServiceFailure;		                    
    
    /**
     * This function is used when the enforcer has only  partial
     * information about an action and needs to determine what range
     * of a property can be allowed by the policy.
     *
     * The enforcer partially fills an ActionInstanceDescription object
	 * and sends it to the PolicyDisclosure object to find those policies that 
	 * are applicable to this action and contain the given property.
	 * PolicyDisclosure will then select only those values for the given property 
	 * that will not conflict with higher priority policies containing
	 * the given property.
     *
     * This is a complex function and needs to be understood in the
     * context of how it is used - this is getting written elsewhere
     * and will be included here at a later time.
     *      
     * @param  origPropertyName String specifying the property for which
	 *						    values are to be found.
	 *  
	 * @param  origActionDesc   ActionInstanceDescription object which will
	 *							be used to find applicable policies.
	 
	 * @param  allTargetValues  Set of possible property values - now it is a fake argument,
	 *                          which should really be calculated by the directory service
	 *                          and passed to the entity disclosing the policy (PolicyDisclosure).
	 *
	 * @param  tight            boolean, if set to 'true', will result in returning
	 *                          only these values for the missing specified property
	 *                          which would satisfy some policy if used alone.   
	 *
     *  @param pObserver        PolicyDecisionObserver, an entity interested in receiving updates
	 *  						whenever policy decisions change.
	 *  
	 * @return	Set		        The Set containing allowed values for the given property,
	 *							or null, if a thread waiting to complete execution
	 *                          of this method has been interrupted.  
	 *                          The interrupted status will be propagated.
	 *                           
	 * @throws ServiceFailure 	when the policy service cannot be reached, or fails.
	 *                         
     */
    public Set getAllowableValuesForActionProperty(String origPropertyName,
                                                   ActionInstanceDescription origActionDesc,
                                                   Set allTargetValues,
                                                   boolean tight,
                                                   PolicyDecisionObserver pObserver)
      throws ServiceFailure;
      
   /** At the moment this method would work only for A- policies for 'moveTo'
	 *  action, and for 'movingTo' property. For other policies, the behaviour
	 *  of this method is undefined. This method would be made more generic
	 *  in future, for handling nested property values.
	 * 
	 *  @param		origPropertyName		the property for which to find those values.
	 * 
	 *  @param		origActionDesc			the AID for which to find the values.
	 *
     *  @param 		pObserver        		PolicyDecisionObserver, an entity interested in receiving updates
	 *  									whenever policy decisions change.
	 *  
	 *	@return		Vector					a Vector of OntInstanceDescriptionImpl objects,
	 *										which are the forbidden values.	
	 *
	 * @throws ServiceFailure 				when the policy service cannot be reached, or fails. 
	 */
	
	public Vector getForbiddenValuesForActionProperty(String origPropertyName,
													  ActionInstanceDescription origActionDesc,
													  PolicyDecisionObserver pObserver)
	throws ServiceFailure;
	

}
