/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/12/10 05:55:11 $
 */
 
package kaos.core.csi.policy;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.policy.ObligationViolationException;

import javax.agent.service.ServiceFailure;

import java.util.Vector;


/**
 * Defines the operations to check permissions and get obligations.
 * 
 * @author KAoS Team
 * $Revision: 1.13 $
 */
public interface PolicyChecking    
{
    /**
     * The method checks if the given action is permitted according to the current set of policies.
     *
     * @param  aid					ActionInstanceDescription describing action
     *
     * @param pObserver        		PolicyDecisionObserver, an entity interested in receiving updates
	 *  							whenever policy decisions change.
	 *  
     * @throws                      KAoSSecurityException if the action is not allowed.
     *
     * @throws                      NullPointerException if the ActionInstanceDescription argument is null.
     *
     * @throws                      ServiceFailure if the policy service is not available or fails.
     */
    public void checkPermission (ActionInstanceDescription aid,
    							 PolicyDecisionObserver pObserver) 
    throws KAoSSecurityException, NullPointerException, ServiceFailure;

    /**
     * The method checks if the given action is permitted according to the current set of policies.
     * However, if the given action is not permitted by a policy, instead of
     * stopping there, it goes on to find all those policies that forbid the
     * given action, and then returns a vector that contains their ids.  
     * The vector containing forbidding policies' ids is stored in the KAoSSecurityException.
     *
     * @param  aid            	ActionInstanceDescription to be checked by the Guard in order to allow/disallow
     *                         	an action. 
     *
     * @param pObserver        	PolicyDecisionObserver, an entity interested in receiving updates
	 *  						whenever policy decisions change.
	 *                                  
     * @throws                	NullPointerException if the aid argument is null.
     *
     * @throws                	ServiceFailure if the policy service is not available, or fails.
     * 
     * @throws					KAoSSecurityException if the action is not permitted.
     * 				
     */
	public void checkDeepPermission (ActionInstanceDescription aid,
									 PolicyDecisionObserver pObserver)
	throws NullPointerException, ServiceFailure, KAoSSecurityException;
	
	/**
     * The method checks if the given action is permitted according to the current set of policies,
     * or if the given action is a trigger for some obligation policy.
     * 
     * If the action is not permitted, the method throws an exception, but if
     * the action is a trigger, the method returns a Vector containing all
     * those obligation policies for which the action is a trigger.
     *
     * @param  aid              ActionInstanceDescription to be checked by the Guard in order to allow/disallow
     *                          the action; or the trigger to be checked for obligation policies.
     *
     * @param pObserver        	PolicyDecisionObserver, an entity interested in receiving updates
	 *  						whenever policy decisions change.
	 *                                  
     * @throws                	ObligationViolationException some aspect of the returned obligations
     * 							have been violated
     *
     * @throws                	ServiceFailure if the policy service is not available, or fails.
     * 
     * @throws					KAoSSecurityException if the action is not permitted.
     * 				
     */
	public void findPolicyDecision (ActionInstanceDescription aid,
									PolicyDecisionObserver pObserver)
	throws ObligationViolationException, KAoSSecurityException, ServiceFailure;
	
    /** 
     * Based on the specified trigger ActionInstanceDescription, select all matching obligation
     * policies and create ActionInstanceDescription describing obligations. 
     * Sort the obligation ActionInstanceDescriptions in descending order by the priority of the obligation policy, which 
     * matched the trigger.
     *
     * @param triggerAID     ActionInstanceDescription describes an instance of the trigger condition sent
     *                       by the trigger condition monitor.
     *
     * @param pObserver    	 PolicyDecisionObserver, an entity interested in receiving updates
	 *  					 whenever policy decisions change.
	 *  
 	 * @return               A Vector of control ActionInstanceDescriptions, whose BasicActionDescriptions are contained 
     *                       in policies matching the parameter triggerAID. The returned control ActionInstanceDescriptions 
     *                       are sorted in descending sequence by their policy priority.
	 *   
     * @throws               ObligationViolationException when specified obligation constraints in a policy applicable 
     *                       to the given trigger are not satisfied.
     *
     * @throws               ServiceFailure if the policy service is not available.
     */
    public Vector getObligationsForTriggerCondition(ActionInstanceDescription triggerAID,
    												PolicyDecisionObserver pObserver) 
    throws ObligationViolationException, ServiceFailure;
}
