/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2005/11/28 21:59:11 $
 */
 
package kaos.core.csi.query;

import kaos.core.service.boot.KAoSBootstrapper;
import kaos.core.service.directory.KAoSAgentDirectoryServiceProxy;
import kaos.ontology.util.QueryFailure;
import kaos.core.util.Logger;
import kaos.ontology.management.UnknownConceptException;

import javax.agent.service.directory.DirectoryFailure;
import java.util.Set;

/**
 * Defines the operations to query the different aspects of domain and actor registrations
 * 
 * @author KAoS Team
 * $Revision: 1.5 $
 */
public class QueryStateImpl
    implements QueryState
{
    private static final QueryStateImpl __INSTANCE = new QueryStateImpl() ;
    private static boolean __INITIALIZED = false;
        
    private KAoSAgentDirectoryServiceProxy      _dirProxy;
	private Logger                              _logger = Logger.getLogger(this); 
	
	/**
     * Retrieve an instance of QueryState.
     *
     * @param transportName String specifying the name of the transport to use for communication with the KAoS Directory Service.
     * 
     * @return a singleton, QueryState.
     */
    public static QueryState getInstance(String transportName)
    throws Exception
    {
        if ( __INITIALIZED ) return __INSTANCE;
        
            __INSTANCE.initialize(transportName);
            __INITIALIZED = true;
            return __INSTANCE;
    }
    
    private QueryStateImpl()
    {
    }

    /*****************************************************************************************
       kaos.core.csi.query.QueryState interface implementation - begin
    *****************************************************************************************/
    /**
     * Test if the <a href="http://www-ksl.stanford.edu/knowledge-sharing/kif/">KIF</a> statemant is true according to the ontologies in the ontology repository.
     *
     * @param statement        to be tested for truth
     *
     * @return                 true if the statement is true and false in the opposite case
     * 
     * @throws                 DirectoryFailure if the connection to the DS cannot be established
     *
     * @throws                 QueryFailure if query failed
     */
    public boolean testTrue (String statement) throws DirectoryFailure, QueryFailure
    {
        boolean result = _dirProxy.testTrue(statement);
        
        return result;
    }
    
    /**
     * Get values of the selected variable satisfing the <a href="http://www-ksl.stanford.edu/knowledge-sharing/kif/">KIF</a> 
     * query with <b>possibly many variable</b>.
     *
     * @param variableName             the name of the selected variable
     * 
     * @param multiplyVariablesQuery   the query
     *
     * @return                         a set with values satisfing the query
     *
     * @throws                 		   DirectoryFailure if the connection to the DS cannot be established
     *
     * @throws                         QueryFailure if query failed.
     */
    public Set askQuery (String variableName, String multiplyVariablesQuery) throws DirectoryFailure, QueryFailure
    {
        Set result = _dirProxy.askQuery(variableName, multiplyVariablesQuery);
        
        if ( result == null ) 
            throw new DirectoryFailure();
        
        return result;
    }
    
    /**
     * Get set of instances of the given concept; if the concept is an instance itself the set contains just this instance.
     *
     * @param conceptName       The name of the concept
     *
     * @return                  Set of instance ids
     *
     * @throws                  DirectoryFailure if the connection to the DS cannot be established
     * 
     * @exception               UnknownConceptException if conceptName is unknow
     */
    public Set getInstancesOf (String conceptName) throws DirectoryFailure, UnknownConceptException
    {
        Set result = _dirProxy.getInstancesOf(conceptName);
        if ( result == null ) 
            throw new DirectoryFailure();
        
        return result;
    }
    
    /**
     * Get values of the variable satisfing the <a href="http://www-ksl.stanford.edu/knowledge-sharing/kif/">KIF</a> query: <b>(property subject ?val)</b>.
     *
     * @param property         the name of the property
     *
     * @param subject          the name of the subject
     *
     * @return                 a set with values for the property on this subject
     * 
     * @throws                 DirectoryFailure if the connection to the DS cannot be established
     *
     * @throws                 QueryFailure if query failed
     */
    public Set getValuesForProperty (String property, String subject) throws DirectoryFailure, QueryFailure
    {
        Set result = _dirProxy.getValuesForProperty(property, subject);
        if ( result == null ) 
            throw new DirectoryFailure();
        
        return result;
    }
    
     /**
     * Get a single value of the variable satisfing the <a href="http://www-ksl.stanford.edu/knowledge-sharing/kif/">KIF</a> 
     *                                                  query: <b>(property subject ?val)</b>.
     *
     * @param property         String containing the name of the property.
     *
     * @param subject          String containing the name of the subject.
     *
     * @return                 String containing the single value for the property on this subject.
     *
     * @throws                 DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                 QueryFailure if reasoning failed.
     *
     */
    public String getUniqueValueForProperty (String property, String subject) 
        throws DirectoryFailure, QueryFailure
    {
        String result = null;
        try {
            result = _dirProxy.getUniqueValueForProperty(property, subject);
        }
        catch(QueryFailure qf){
            throw qf;
        }
         catch(DirectoryFailure dfe){
            throw  dfe;
        }
        
        if ( result == null ) 
            throw new DirectoryFailure();
        
        return result;
    }
    
    /**
     * Finds the ultimate Ontology type of the provided individual
     *
     * @param individualName           the name of the individual
     *
     * @return                         the set of names of the Ontology classes being the most narrow ones for the given individual
     * 
     * @throws DirectoryFailure 
     *
     * @throws UnknownConceptException if concept unknow or any problem with JTP approached
     */
    public Set getUltimateOntologyTypesOfIndividual(String individualName) throws DirectoryFailure, UnknownConceptException
    {
        Set result = _dirProxy.getUltimateOntologyTypesOfIndividual(individualName);
        if ( result == null ) 
            throw new DirectoryFailure();
        
        return result;
    }

    /*****************************************************************************************
       kaos.core.csi.query.QueryState interface implementation - end
    *****************************************************************************************/
    /////////////////////////////   PRIVATE METHODS BEGIN
    /*
     **/
    private void initialize(String transportName)
    throws Exception
    {
        try {
            _dirProxy = KAoSBootstrapper.getKAoSDirectoryProxyForTransport(transportName);
        }
        catch(Exception e) {
            e.printStackTrace();
            throw new Exception("    \n\n    *** QueryStateImpl: unable to obtain a reference the DirectoryService for transport: "
                                + transportName + "!");
        }
    }
    //////////////////////////// PRIVATE METHODS END
    // Utility methods ///////////////////////////////////////////
      
    protected void logMessage(String msg, int logLevel)
    {
   		if (_logger.isEnabledFor(logLevel))
   			_logger.logMessage(msg, logLevel);
    }
	
    protected void logMessage(String msg, Exception e, int logLevel)
    {
   		if (_logger.isEnabledFor(logLevel))
   			_logger.logMessage(msg, e, logLevel);
    }
 }
