/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/10/27 18:43:52 $
 */
 
package kaos.core.csi.registration;

import kaos.core.service.directory.ModificationException;
import kaos.core.service.directory.EntityHostEnvDescription;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.policy.guard.GuardInstantiationException;
import kaos.spatial.*;

import javax.agent.JasBean;
import javax.agent.service.directory.AgentDescription;
import javax.agent.service.directory.AlreadyRegisteredException;
import javax.agent.service.directory.DirectoryFailure;
import javax.agent.service.directory.NotRegisteredException;

import java.util.List;
import java.util.Set;
import java.util.Vector;

/**
 * Defines the operations to register and deregister entities and entity hosting environments
 * in the KAoS Directory Service and operations to modify the registration information.
 * 
 * @author KAoS Team
 * $Revision: 1.8 $
 */
public interface DirectoryRegistration
{
   /** 
    * Register domain with the given name in the KAoS Directory Service. 
    *
    * @param domainName         String containing the name of domain to register.
    *
    *
    * @throws                   DirectoryFailure if the connection to the DS cannot be established.
    *
    * @throws                   AlreadyRegisteredException if the registering domain has been already registered in the DS 
    *
    */
   public void registerDomain(String domainName)
   throws DirectoryFailure,
          AlreadyRegisteredException;

   /** 
    * Register the entity described by the given AgentDescription in the KAoS Directory Service. 
    *
    * @param entityDescription  AgentDescription is the JAS AgentDescription, whose subclasses describe the various 
    *                           types of entities to be registered in KAoS:  agents, guards, domains, containers. 
    *                           Those entities are subjects to policies or potential subjects to policies, 
    *                           or the entities are  using KAoS for messaging.  
    *
    * @param guarded            boolean indicating the guarded/unguarded status for agents and containers.  
    *                           If the entity is to be guarded, the implementation of this method will instantiate a guard, 
    *                           or check if a guard already exists and proceed with the registration via the guard.
    *
    * @param replace            boolean indicating whether to replace existing AgentDescription.
    *
    * @throws                   DirectoryFailure if the connection to the DS cannot be established.
    *
    * @throws                   AlreadyRegisteredException if the registering entity has been already registered in the DS 
    *                           (if the replace = false and there is already registration for the specified AgentDescription).
    *
    */
   public void registerEntity(AgentDescription entityDescription,
                              boolean guarded, 
                              boolean replace)
   throws DirectoryFailure,
          AlreadyRegisteredException,
          GuardInstantiationException;
          
   /** 
    * Register the entity described by the given AgentDescription and the List of Properties in the KAoS Directory Service. 
    *
    * @param entityDescription  AgentDescription is the JAS AgentDescription, whose subclasses describe the various 
    *                           types of entities to be registered in KAoS:  agents, guards, domains, containers. 
    *                           Those entities are subjects to policies or potential subjects to policies, 
    *                           or the entities are  using KAoS for messaging.  
    *
    * @param properties         List of Property objects further describing the registering entity.
    *
    * @param guarded            boolean indicating the guarded/unguarded status for agents and containers.  
    *                           If the entity is to be guarded, the implementation of this method will instantiate a guard, 
    *                           or check if a guard already exists and proceed with the registration via the guard.
    *
    * @param replace            boolean indicating whether to replace existing AgentDescription.
    *
    * @throws                   DirectoryFailure if the connection to the DS cannot be established.
    *
    * @throws                   AlreadyRegisteredException if the registering entity has been already registered in the DS 
    *                           (if the replace = false and there is already registration for the specified AgentDescription).
    *
    */
   public void registerEntityWithProperties(AgentDescription entityDescription,
                                            List properties,
                                            boolean guarded, 
                                            boolean replace)
   throws DirectoryFailure,
          AlreadyRegisteredException,
          GuardInstantiationException;
          
          
  /** 
    * Register the entity described by the given AgentDescription and the spatial properties in the KAoS Directory Service
    * *AND* the SpatialReasoner. 
    *
    * @param entityDescription  AgentDescription is the JAS AgentDescription, whose subclasses describe the various 
    *                           types of entities to be registered in KAoS:  agents, guards, domains, containers. 
    *                           Those entities are subjects to policies or potential subjects to policies, 
    *                           or the entities are  using KAoS for messaging.  
    *
    * @param objectLocation     Location3D containing the X,Y,Z coordinates of the registering entity.
    *
    * @param objectOrientation  Orientation3D containing the X,Y,Z radius of the registering entity.
    *
    * @param objectOccupiedSpace    Block3D containing corner locations determining the occupied space of the registering entity.
    *
    * @param guarded            boolean indicating the guarded/unguarded status for agents and containers.  
    *                           If the entity is to be guarded, the implementation of this method will instantiate a guard, 
    *                           or check if a guard already exists and proceed with the registration via the guard.
    *
    * @param replace            boolean indicating whether to replace existing AgentDescription.
    *
    * @throws                   DirectoryFailure if the connection to the DS cannot be established.
    *
    * @throws                   AlreadyRegisteredException if the registering entity has been already registered in the DS 
    *                           (if the replace = false and there is already registration for the specified AgentDescription).
    *
    */
   public void registerEntityWithSpatialProperties(AgentDescription entityDescription,
                                                   Location3D objectLocation, 
                                                   Orientation3D objectOrientation,
                                                   Block3D objectOccupiedSpace,
                                                   boolean guarded, 
                                                   boolean replace)
   throws DirectoryFailure,
          AlreadyRegisteredException,
          GuardInstantiationException;


   /** 
    * Register object identified by the given objectID and its spatial properties in the SpatialReasoner.
    * Important:  Note that the object is *not* registered in the KAoS DirectoryService, only in the SpatialReasoner.
    *             If you want to register a given object in the KAoS Directory service, use method registerEntityWithSpatialProperties(),
    *             which will do both, register in the KAoS DS and the SpatialReasoner.
    *
    * @param objectID               String uniquely identifying registering object in the SpatialReasoner.
    *
    * @param objectOntologyType     Set containing Strings describing the ontological types of the registering object in the SpatialReasoner.
    *
    * @param objectLocation         Location3D containing the X,Y,Z coordinates of the registering object.
    *
    * @param objectOrientation      Orientation3D containing the X,Y,Z radius of the registering object.
    *
    * @param objectOccupiedSpace    Block3D containing corner locations determining the occupied space of the registering object.
    *
    *
    * @throws                   DirectoryFailure if the connection to the SpatialReasoner cannot be established.
    *
    * @throws                   AlreadyRegisteredException if the registering object has been already registered in the SpatialReasoner. 
    *
    */
   public void registerObjectWithSpatialProperties(String objectID, 
                                                   Set objectOntologyType,
                                                   Location3D objectLocation, 
                                                   Orientation3D objectOrientation,
                                                   Block3D objectOccupiedSpace)
   throws DirectoryFailure,
          AlreadyRegisteredException;


   /** 
    * Register the entity hosting environment described by the given EntityHostEnvDescription in the KAoS Directory Service. 
    *
    * @param eheDescription     EntityHostEnvDescription describes the hosting environment for agents and guards.
    *
    * @param guarded            boolean indicating the guarded/unguarded status for the environment. 
    *                           If the hosted environemnt is to be guarded, the implementation of 
    *                           this method will instantiate a guard, or check if a guard already exists 
    *                           and proceed with the registration via the guard.
    *
    * @param replace            boolean indicating whether to replace existing EntityHostEnvDescription.
    *
    * @throws                   DirectoryFailure if the connection to the DS cannot be established.
    *
    * @throws                   AlreadyRegisteredException if the registering hosting environment has been already registered in the DS 
    *                           (if the replace = false and there is already registration for the specified EntityHostEnvDescription).
    *
    */
   public void registerEntityHostingEnv(EntityHostEnvDescription eheDescription,
                                        boolean guarded, 
                                        boolean replace)
   throws DirectoryFailure,
          AlreadyRegisteredException,
          GuardInstantiationException;
          
          
   /** 
    * Modify properties of the entity identified by the given entityGUID.
    *
    * @param entityGUID     String containing the entity GUID.
    *
    * @param properties     List of Properties to be modified.  Each Property contains the name, value, value description, 
    *                       modification type (add/remove/set, etc), modification result.  
    *                       Refer to the Property and ModificationResult interfaces for detailed description.
    *
    * @throws               DirectoryFailure if the connection to the DS cannot be established.
    *
    * @throws               NotRegisteredException if the entity whose properties are to be modified 
    *                       has not been registered in the DS.
    *
    * @throws               ModificationException if one or more property update failed.  
    *
    */
   public void modifyEntityProperties(String entityGUID, List properties)
   throws DirectoryFailure,
          NotRegisteredException,
          ModificationException;
   
    /** 
     * Modify given spatial properties of the entity identified by the given entityGUID in the KAoS Directory Service *AND*
     * the SpatialReasoner.
     *
     * @param entityGUID            String containing the entity GUID.
     *
     * @param objectLocation        modified Location3D containing the X,Y,Z coordinates of the entity.
     *
     * @param objectOrientation     modified Orientation3D containing the X,Y,Z radius of the entity.
     *
     * @param objectOccupiedSpace   modified Block3D containing corner locations determining the occupied space of the entity.
     *
     * @throws                      DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                      NotRegisteredException if the entity whose properties are to be modified 
     *                              has not been registered in the DS.
     *
     * @throws                      ModificationException if one or more property update failed.  
     *
     */
    public void modifyEntitySpatialProperties(String entityGUID,  
                                              Location3D objectLocation, 
                                              Orientation3D objectOrientation,
                                              Block3D objectOccupiedSpace)
    throws DirectoryFailure,
           NotRegisteredException,
           ModificationException;
     
           
   /** 
    * Modify spatial properties of the object identified by the given objectID in the SpatialReasoner.
    * Important:  Note that the properties are *not* modified in the KAoS DirectoryService, only in the SpatialReasoner.
    *             If you want to modify properties of a given object in the KAoS Directory service, use method modifyEntitySpatialProperties(),
    *             which will do both, modify in the KAoS DS and the SpatialReasoner.
    *
    * @param objectID               String uniquely identifying object, whose spatial properties will be modified in the SpatialReasoner.
    *
    * @param objectLocation         modified Location3D containing the X,Y,Z coordinates of the entity.
    *
    * @param objectOrientation      modified Orientation3D containing the X,Y,Z radius of the entity.
    *
    * @param objectOccupiedSpace    modified Block3D containing corner locations determining the occupied space of the entity.
    *
    * @throws                       DirectoryFailure if the connection to the SpatialReasoner cannot be established.
    *
    * @throws                       NotRegisteredException if the object, whose properties are to be modified has not been registered 
    *                               in the SpatialReasoner.
    *
    */
   public void modifyObjectSpatialProperties(String objectID, 
                                             Location3D objectLocation, 
                                             Orientation3D objectOrientation,
                                             Block3D objectOccupiedSpace)
    throws DirectoryFailure,
           NotRegisteredException;
           
   /** 
    * Deregister the entity described by the given AgentDescription from the KAoS Directory Service. 
    *
    * @param entityDescription  AgentDescription of the entity to be deregistered.
    *
    * @throws                   DirectoryFailure if the connection to the DS cannot be established.
    *
    * @throws                   NotRegisteredException if the entity to be deregistered has not been registered in the DS.
    *
    */
   public void deregisterEntity(AgentDescription entityDescription)
   throws DirectoryFailure,
          NotRegisteredException;
          
    /** 
     * Deregister the entity hosting environment described by the given EntityHostEnvDescription from the KAoS Directory Service. 
     *
     * @param hostEnvDescription    EntityHostEnvDescription of the entity hosting environment to be deregistered.
     *
     * @throws                      DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                      NotRegisteredException if the entity to be deregistered has not been registered in the DS.
     *
     */
    public void deregisterEntityHostingEnv(EntityHostEnvDescription  hostEnvDescription)
    throws DirectoryFailure,
          NotRegisteredException;
          
    /** 
     * Create an instance of the KAoSAgentDescription using the given parameters.
     *
     * @param guid              String containing the globally unique identifier for the agent, or  null. If null is passed, then
     *                          the Agent Naming Service will create the identifier.
     *
     * @param nickname          String containing agent's nickname.
     *
     * @param domainNames       Vector containing the names of the domains the agent will become a member of.
     *
     * @param transportName     String specifying the name of the transport the agent wants to use, or null.
     *                          If null is specified, then the Locators will be empty.
     *
     */
    public KAoSAgentDescription createKAoSAgentDescription(String guid,
                                                           String nickname,
                                                           Vector domainNames,
                                                           String transportName,
                                                           JasBean agentInitInfo);
                                                           
 }
