/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/11/23 17:49:07 $
 */
package kaos.core.csi.registration;

import java.io.Serializable;
import java.rmi.dgc.VMID;

/**
 * $Revision: 1.9 $
 */
public class PropertyImpl
    implements Property
   {
    /**
	 * 
	 */
	private static final long serialVersionUID = 1155439233679521374L;
	
	protected String                    _name;
    protected String                    _description;
    protected ModificationResult        _modResult;
    protected int                       _scope;
    protected Serializable              _value;
    protected Serializable              _propertyObj;
    private String                      _id;

    /**
     * Specifies operations used to update entity properties. In addition to the property name, value, description and
     * modification type, operations include also accessing modification result.
     */
    public PropertyImpl()
    {
        initID();
    }

    public PropertyImpl(String name,
                        String description,
                        ModificationResult modResult,
                        int scope,
                        Serializable value)
    {
        if ( name != null ) _name = name;
        if ( description != null ) _description = description;
        if ( modResult != null ) _modResult = modResult;
        if ( value != null ) _value = value;
        setScope(scope);
        initID();
    }
    
    //////////////////////////////////////////////////////////////
    // Implementation of the Property interface
   
    /** 
     * Set the property name.
     *
     * @param name      String containing the name of the property. 
     */
    public void setName(String name)
    {
        _name = name;
    }
   
    /**
     * Obtain the property name.
     *
     * @return String containing the name of the property. 
     */
    public String  getName()
    {
        return _name;
    }
    
    /** 
     * Set the result of the property modification.
     *
     * @param result    ModificationResult describing the result of modifying the property.
     */
    public void setModificationResult(ModificationResult result)
    {
        _modResult = result;
    }
   
    /**
     * Obtain the result of the property modification.
     *
     * @return ModificationResult describing the result of modifying the property.
     */
    public ModificationResult getModificationResult()
    {
        return _modResult;
    }
    
    /** 
     * Set the new value of the property.
     *
     * @param value      Serializable containing the new valueof the property. 
     */
    public void setValue(Serializable value)
    {
        _value = value;
    }
   
    /**
     * Obtain the new value of the property.
     *
     * @return Serializable containing the new value of the property.
     */
    public Serializable  getValue()
    {
        return _value;
    }
    
    /** 
     * Set the OWL description of the property to be modified. 
     *
     * @param desc   String containing the OWL description of the property to be modified.
     */
    public void setDescription(String desc)
    {
        _description = desc;
    }
    
    /** 
     * Return the OWL description of the property to be modified.
     *
     * @return          String containing the OWL description of the property to be modified
     *               
     */
    public String  getDescription()
    {
        return _description;
    }
   
    /** 
     * Set the scope of the modification:
     * 0 = PUBLIC scope indicates updates to the Directory Service 
     * 1 = PRIVATE scope indicates local updates only
     *
     * @param scope       int specifying the scope of the property update.
     *               
     */
    public void setScope(int scope)
    {
        if ( ( scope == 0 ) || ( scope == 1 )){
            _scope = scope;
        }
        else {// set default PUBLIC
            _scope = 0;
        }
            
    }
    
    /** 
     * Obtain the scope of the modification:
     * 0 = PUBLIC scope indicates updates to the Directory Service 
     * 1 = PRIVATE scope indicates local updates only
     *
     * @return       int specifying the scope of the property update.
     *               
     */
    public int getScope( )
    {
        return _scope;
    }

    /** 
     * Obtain the unique sequence id of this property.
     *
     * This id is used in remote updates of this property in the DS.  If the property
     * update failes, then the Property is returned back to the requestor
     * and the requestor can identify the failed property and not update its local
     * collection of Properties.
     *
     * @return       String containing the unique sequence id of this Property.
     *               
     */
    public String getSequenceId()
    {
        return _id;
    }

    /** 
     * Return a Serializable, encapsulating a property, which this Property represents.  
     *
     * This method is used to manipulate spatial properties, ex. Location3D, Orientation3D, etc.
     *
     * @return       Serializable containing a property.
     *               
     */
    public Serializable getPropertyObject()
    {
        return _propertyObj;
    }
    
    /** 
     * Set the given Serializable property, which this Property represents.  The Serializable
     * may have its own API.
     *
     * @param property       Serializable encapsulating a property.
     *               
     */
    public void setPropertyObject(Serializable property)
    {
        _propertyObj = property;
    }
    
    /**
     * A utility method to create a String representation of the Property. 
     *
     * @return   String representing the Property.
     */
    public String toString()
    {
        StringBuffer buffer = new StringBuffer(); 
        buffer.append("\nName: " + _name + "\n"); 
        buffer.append("Value: " + _value + "\n"); 
        buffer.append("Description: " + _description + "\n"); 
        buffer.append("PropertyObject: " + _propertyObj + "\n"); 
        buffer.append("Scope: " + _scope + "\n"); 
        buffer.append("ModificationResult: " + _modResult + "\n"); 
        
        return buffer.toString(); 
    }
    
    private void initID()
    {
        _id = new VMID().toString();
    }
   }