/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2005/11/28 21:59:11 $
 */
package kaos.core.csi.registration;

import kaos.core.csi.CSIFactory;
import kaos.core.csi.policy.enforcement.EnforcerInstantiationException;
import kaos.core.csi.transport.TransportSupport;
import kaos.core.service.KAoSServiceRoot;
import kaos.core.service.boot.KAoSBootstrapper;
import kaos.core.service.directory.DefaultDomainDescription;
import kaos.core.service.directory.DefaultKAoSAgentDescription;
import kaos.core.service.directory.DefaultKAoSEntityDescription;
import kaos.core.service.directory.DomainDescription;
import kaos.core.service.directory.EntityHostEnvDescription;
import kaos.core.service.directory.HasParentException;
import kaos.core.service.directory.ModificationException;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.core.service.directory.KAoSDirectoryService;
import kaos.core.service.directory.KAoSAgentDirectoryServiceProxy;
import kaos.core.service.directory.KAoSEntityDescription;
import kaos.core.service.naming.KAoSAgentNamingService;
import kaos.core.util.Logger;
import kaos.core.util.KAoSConstants;
import kaos.policy.guard.Guard;
import kaos.policy.guard.GuardRetriever;
import kaos.policy.guard.GuardInstantiationException;
import kaos.ontology.management.UnknownConceptException;
import kaos.ontology.vocabulary.DefaultNamespaces;
import kaos.ontology.vocabulary.PlaceConcepts;
import kaos.spatial.Location3D;
import kaos.spatial.Orientation3D;
import kaos.spatial.Block3D;
import kaos.spatial.SpatialReasoner;

import kaos.spatial.vocabulary.*;


import javax.agent.AgentName;
import javax.agent.JasBean;
import javax.agent.Locator;
import javax.agent.service.directory.AgentDescription;
import javax.agent.service.directory.AlreadyRegisteredException;
import javax.agent.service.directory.DirectoryFailure;
import javax.agent.service.directory.NotRegisteredException;
import javax.agent.service.naming.AgentNamingService;

import ri.acr.AcrAgentName;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;


/**
 * This class implements the Registration interface, which efines the operations to register and deregister entities and entity hosting environments
 * in the KAoS Directory Service and operations to modify the registration information.  It also defines operations to
 * register entitites in KAoS Domains and to maintain domain hierarchy.
 * $Revision: 1.16 $
 */
public class RegistrationImpl
    implements Registration
               
{
    private static final RegistrationImpl __INSTANCE = new RegistrationImpl() ;
    private static boolean __INITIALIZED = false;
        
    private KAoSAgentDirectoryServiceProxy      _dirProxy;
    private String                              _transportName;
	private Logger                              _logger = Logger.getLogger(this);
            
    
    /**
     * Retrieve an instance of Registration.
     *
     * @param transportName String specifying the name of the transport to use for communication with the KAoS Directory Service.
     * 
     * @return a singleton, Registration.
     */
    public static Registration getInstance(String transportName)
    throws Exception
    {
        if ( __INITIALIZED ) return __INSTANCE;
        
            __INSTANCE.initialize(transportName);
            __INITIALIZED = true;
            return __INSTANCE;
    }
    
    /**
     * Retrieve an instance of Registration.
     *
     * @return a singleton, Registration.
     */
    public static Registration getInstance()
    throws Exception
    {
        if ( __INITIALIZED ) return __INSTANCE;
        
        String transportName = System.getProperty(KAoSConstants.DIRECTORY_TRANSPORT);
        if ( transportName == null ) throw new Exception("System property: directory.transport is null, unable to connect to the Directory Service!");
        
        return getInstance(transportName);
    }
    
    private RegistrationImpl()
    {
    }

    /*****************************************************************************************
       kaos.core.csi.registration.DirectoryRegistration interface implementation - begin
    *****************************************************************************************/

   /** 
    * Register domain with the given name in the KAoS Directory Service. 
    *
    * @param domainName         String containing the name of domain to register.
    *
    *
    * @throws                   DirectoryFailure if the connection to the DS cannot be established.
    *
    * @throws                   AlreadyRegisteredException if the registering domain has been already registered in the DS 
    *
    */
    public void registerDomain(String domainName)
    throws DirectoryFailure,
          AlreadyRegisteredException
    {
         DefaultDomainDescription desc = new DefaultDomainDescription();
         desc.setDomainName(domainName);
         try {
            registerEntity(desc,
                           false, 
                           false);
         }
         catch(DirectoryFailure df){
            throw df;
         }
         catch(AlreadyRegisteredException are){
            throw are;
         }
         catch(GuardInstantiationException gie){
            //ignore since this should not happen
         }
    }
    
    /** 
     * Register the entity described by the given AgentDescription in the KAoS Directory Service. 
     *
     * @param entityDescription  AgentDescription is the JAS AgentDescription, whose subclasses describe the various 
     *                           types of entities to be registered in KAoS:  agents, guards, domains, containers. 
     *                           Those entities are subjects to policies or potential subjects to policies, 
     *                           or the entities are  using KAoS for messaging.  
     *
     * @param guarded            boolean indicating the guarded/unguarded status for agents and containers.  
     *                           If the entity is to be guarded, the implementation of this method will instantiate a guard, 
     *                           or check if a guard already exists and proceed with the registration via the guard.
     *
     * @param replace            boolean indicating whether to replace existing AgentDescription.
     *
     * @throws                   DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                   AlreadyRegisteredException if the registering entity has been already registered in the DS 
     *                           (if the replace = false and there is already registration for the specified AgentDescription).
     *
     */
    public void registerEntity(AgentDescription entityDescription,
                               boolean guarded, 
                               boolean replace)
    throws DirectoryFailure,
           AlreadyRegisteredException,
           GuardInstantiationException
    {       
        entityDescription.set(KAoSDirectoryService.REGISTRATION_REPLACE_FLAG,new Boolean(replace));
        //Check, if the agent is guarded.
		if ( guarded ) {
            Guard guard = GuardRetriever.getGuard();
            KAoSAgentDescription desc = null;
            if ( entityDescription instanceof KAoSAgentDescription ) {
                desc = (KAoSAgentDescription)entityDescription;
            }
            else {
                throw new IllegalArgumentException("AgentDescription not instance of KAoSAgentDescription - Guard unable to register entity!" );
            }
                
            if ( guard == null ) {
                List domainNames = null;
                
                domainNames = desc.getDomainNames();
                
                Vector domainNamesV;
                if ( domainNames == null ) {
                    domainNamesV = new Vector();
                }
                else{
                   domainNamesV = new Vector(domainNames); 
                }
                
		        guard = KAoSBootstrapper.instantiateGuard(_transportName,
                                                          domainNamesV);
            }
            try {                                         
                guard.newAgent(desc, null);
            }
            catch (AlreadyRegisteredException are){
                throw are;
            }
            catch (DirectoryFailure df){
                throw df;
            }
            catch(EnforcerInstantiationException eie){
                // TO DO: check how to handle this exception.  Is it ever thrown?
                eie.printStackTrace();
            }
            catch(UnknownConceptException uce) {
                uce.printStackTrace();
                // TO DO: check how to handle this exception.  Is it ever thrown?
            }   
           
	    }
	    else {
	        _dirProxy.register(entityDescription);
	    }
		
    }

/** 
    * Register the entity described by the given AgentDescription and the spatial properties in the KAoS Directory Service
    * *AND* the SpatialReasoner. 
    *
    * @param entityDescription  AgentDescription is the JAS AgentDescription, whose subclasses describe the various 
    *                           types of entities to be registered in KAoS:  agents, guards, domains, containers. 
    *                           Those entities are subjects to policies or potential subjects to policies, 
    *                           or the entities are  using KAoS for messaging.  
    *
    * @param objectLocation         Location3D containing the X,Y,Z coordinates of the registering entity.
    *
    * @param objectOrientation      Orientation3D containing the X,Y,Z radius of the registering entity.
    *
    * @param objectOccupiedSpace    Block3D containing corner locations determining the occupied space of the registering entity.
    *
    * @param guarded            boolean indicating the guarded/unguarded status for agents and containers.  
    *                           If the entity is to be guarded, the implementation of this method will instantiate a guard, 
    *                           or check if a guard already exists and proceed with the registration via the guard.
    *
    * @param replace            boolean indicating whether to replace existing AgentDescription.
    *
    * @throws                   DirectoryFailure if the connection to the DS cannot be established.
    *
    * @throws                   AlreadyRegisteredException if the registering entity has been already registered in the DS 
    *                           (if the replace = false and there is already registration for the specified AgentDescription).
    *
    */
   public void registerEntityWithSpatialProperties(AgentDescription entityDescription,
                                                   Location3D objectLocation, 
                                                   Orientation3D objectOrientation,
                                                   Block3D objectOccupiedSpace,
                                                   boolean guarded, 
                                                   boolean replace)
   throws DirectoryFailure,
          AlreadyRegisteredException,
          GuardInstantiationException
    {
        Property locationProp = null;
        Property orientationProp = null;
        Property blockProp = null;
        ArrayList aList = null;
        
        if ( objectLocation != null || objectOrientation != null || objectOccupiedSpace != null )
            aList = new ArrayList();          
            
        if ( objectLocation != null ) {
            locationProp = new PropertyImpl();
            locationProp.setName(PhysicalAddressConcepts.hasLocation());
            locationProp.setValue(objectLocation.getMyId());
            locationProp.setPropertyObject(objectLocation);
            aList.add(locationProp);
        }
        
        if ( objectOrientation != null ) {
            orientationProp = new PropertyImpl();
            orientationProp.setName(PhysicalAddressConcepts.hasOrientation());
            orientationProp.setValue(objectOrientation.getMyId());
            orientationProp.setPropertyObject(objectOrientation);
            aList.add(orientationProp);
        }
        
        if ( objectOccupiedSpace != null ) {
            blockProp = new PropertyImpl();
            blockProp.setName(PlaceConcepts.isEncompassedBy());
            blockProp.setValue(objectOccupiedSpace.getMyId());
            blockProp.setPropertyObject(objectOccupiedSpace);
            aList.add(blockProp);
        }
        
        if ( aList != null ) {
            registerEntityWithProperties(entityDescription,
                                         aList,
                                         guarded, 
                                         replace);
        }
        else {
            registerEntity(entityDescription,
                           guarded, 
                           replace);
            return;
        }
        
        DefaultKAoSEntityDescription kDesc = (DefaultKAoSEntityDescription)entityDescription;
        Set names = new HashSet();
        Enumeration e = kDesc.getEntityOntologicalTypeNames();
        while ( e.hasMoreElements() ) {
            names.add((String) e.nextElement());
        }
        
        registerObjectWithSpatialProperties(kDesc.getEntityNameAsString(),
                                            names,
                                            objectLocation,
                                            objectOrientation,
                                            objectOccupiedSpace);
                                     
    }


   /** 
    * Register object identified by the given objectID and its spatial properties in the SpatialReasoner.
    * Important:  Note that the object is *not* registered in the KAoS DirectoryService, only in the SpatialReasoner.
    *             If you want to register a given object in the KAoS Directory service, use method registerEntityWithSpatialProperties(),
    *             which will do both, register in the KAoS DS and the SpatialReasoner.
    *
    * @param objectID               String uniquely identifying registering object in the SpatialReasoner.
    *
    * @param objectOntologyTypes    Set containing Strings describing the ontological types of the registering object in the SpatialReasoner.
    *
    * @param objectLocation         Location3D containing the X,Y,Z coordinates of the registering object.
    *
    * @param objectOrientation      Orientation3D containing the X,Y,Z radius of the registering object.
    *
    * @param objectOccupiedSpace    Block3D containing corner locations determining the occupied space of the registering object.
    *
    *
    * @throws                   DirectoryFailure if the connection to the SpatialReasoner cannot be established.
    *
    * @throws                   AlreadyRegisteredException if the registering object has been already registered in the SpatialReasoner. 
    *
    */
   public void registerObjectWithSpatialProperties(String objectID, 
                                                   Set objectOntologyTypes,
                                                   Location3D objectLocation, 
                                                   Orientation3D objectOrientation,
                                                   Block3D objectOccupiedSpace)
    throws DirectoryFailure,
           AlreadyRegisteredException
    {
        SpatialReasoner reasoner = null;
        try {
            CSIFactory.setCurrentTransport(_transportName);
            reasoner = CSIFactory.getSpatialReasoner();
        }
        catch(Exception e) {
            e.printStackTrace();
            throw new DirectoryFailure("\n  *** Unable to obtain SpatialReasoner for transport: " + _transportName
                                        + "\n  *** Unable to register object: " + objectID);
        }
        reasoner.registerSpatialObject(objectID, 
                                       objectOntologyTypes,
                                       objectLocation, 
                                       objectOrientation,
                                       objectOccupiedSpace); 
        
    }


   /** 
    * Register the entity described by the given AgentDescription and the List of Properties in the KAoS Directory Service. 
    *
    * @param entityDescription  AgentDescription is the JAS AgentDescription, whose subclasses describe the various 
    *                           types of entities to be registered in KAoS:  agents, guards, domains, containers. 
    *                           Those entities are subjects to policies or potential subjects to policies, 
    *                           or the entities are  using KAoS for messaging.  
    *
    * @param properties         List of Property objects further describing the registering entity.
    *
    * @param guarded            boolean indicating the guarded/unguarded status for agents and containers.  
    *                           If the entity is to be guarded, the implementation of this method will instantiate a guard, 
    *                           or check if a guard already exists and proceed with the registration via the guard.
    *
    * @param replace            boolean indicating whether to replace existing AgentDescription.
    *
    * @throws                   DirectoryFailure if the connection to the DS cannot be established.
    *
    * @throws                   AlreadyRegisteredException if the registering entity has been already registered in the DS 
    *                           (if the replace = false and there is already registration for the specified AgentDescription).
    *
    */
   public void registerEntityWithProperties(AgentDescription entityDescription,
                                            List properties,
                                            boolean guarded, 
                                            boolean replace)
   throws DirectoryFailure,
          AlreadyRegisteredException,
          GuardInstantiationException
    {
        Iterator propertiesIt = properties.iterator();
        while (propertiesIt.hasNext()) {
            Object propObj = propertiesIt.next();
            if ( propObj instanceof Property ) {
                Property prop = (Property) propObj;
                entityDescription.set(prop.getName(), prop);
            }
        }
        registerEntity(entityDescription,
                       guarded,
                       replace);
    }

    /** 
     * Register the entity hosting environment described by the given EntityHostEnvDescription in the KAoS Directory Service. 
     *
     * @param eheDescription     EntityHostEnvDescription describes the hosting environment for agents and guards.
     *
     * @param guarded            boolean indicating the guarded/unguarded status for the environment. 
     *                           If the hosted environemnt is to be guarded, the implementation of 
     *                           this method will instantiate a guard, or check if a guard already exists 
     *                           and proceed with the registration via the guard.
     *
     * @param replace            boolean indicating whether to replace existing EntityHostEnvDescription.
     *
     * @throws                   DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                   AlreadyRegisteredException if the registering hosting environment has been already registered in the DS 
     *                           (if the replace = false and there is already registration for the specified EntityHostEnvDescription).
     *
     */
    public void registerEntityHostingEnv(EntityHostEnvDescription eheDescription,
                                        boolean guarded, 
                                        boolean replace)
    throws DirectoryFailure,
           AlreadyRegisteredException,
           GuardInstantiationException
    {
        System.out.println("    \n\n    TO BE IMPLEMENTED!");
    }
          
          
    /** 
     * Modify properties of the entity identified by the given entityGUID.
     *
     * @param entityGUID     String containing the entity GUID.
     *
     * @param properties     List of Properties to be modified.  Each Property contains the name, value, value description, 
     *                       modification type (add/remove/set, etc), modification result.  
     *                       Refer to the Property and ModificationResult interfaces for detailed description.
     *
     * @throws               DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws               NotRegisteredException if the entity whose properties are to be modified 
     *                       has not been registered in the DS.
     *
     * @throws               ModificationException if one or more property update failed.  
     *
     */
    public void modifyEntityProperties(String entityGUID, List properties)
    throws DirectoryFailure,
          NotRegisteredException,
          ModificationException
    {
        if (  properties == null ) 
            throw new ModificationException("Properties to modify are null! Requested modification failed");
        
        if (  properties.isEmpty() ) return;
        
        _dirProxy.modifyEntityProperties(entityGUID, properties);
    }
   
   /** 
     * Modify given spatial properties of the entity identified by the given entityGUID in the KAoS Directory Service *AND*
     * the SpatialReasoner.
     *
     * @param entityGUID            String containing the entity GUID.
     *
     * @param objectLocation        modified Location3D containing the X,Y,Z coordinates of the entity.
     *
     * @param objectOrientation     modified Orientation3D containing the X,Y,Z radius of the entity.
     *
     * @param objectOccupiedSpace   modified Block3D containing corner locations determining the occupied space of the entity.
     *
     * @throws                      DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                      NotRegisteredException if the entity whose properties are to be modified 
     *                              has not been registered in the DS.
     *
     * @throws                      ModificationException if one or more property update failed.  
     *
     */
    public void modifyEntitySpatialProperties(String entityGUID,  
                                              Location3D objectLocation, 
                                              Orientation3D objectOrientation,
                                              Block3D objectOccupiedSpace)
    throws DirectoryFailure,
           NotRegisteredException,
           ModificationException
    {
        SpatialReasoner reasoner = null;
        try {
            CSIFactory.setCurrentTransport(_transportName);
            reasoner = CSIFactory.getSpatialReasoner();
        }
        catch(Exception e) {
            e.printStackTrace();
            throw new DirectoryFailure("Unable to obtain SpatialReasoner for transport: " + _transportName 
                                        + " Modification of properties for: " + entityGUID + " fails!");
        }
        
        Property locationProp = null;
        Property orientationProp = null;
        Property blockProp = null;
        ArrayList aList = null;
         
        if ( objectLocation != null || objectOrientation != null || objectOccupiedSpace != null )
            aList = new ArrayList();    
            
        if ( objectLocation != null ) {
            if ( objectLocation.getModificationType() == null ) {
                if (_logger.isEnabledFor(Logger.WARN))
                    logMessage(".modifyEntitySpatialProperties: NO MODIFICATION TYPE SPECIFIED FOR LOCATION: " 
                                + objectLocation + "\n  FOR ENTITY: " + entityGUID + " NO UPDATE WILL BE MADE!", Logger.WARN);   
            }
            else {
                locationProp = new PropertyImpl();
                locationProp.setName(PhysicalAddressConcepts.hasLocation());
                locationProp.setValue(objectLocation.getMyId());
                locationProp.setPropertyObject(objectLocation);
                ModificationResultImpl result = new ModificationResultImpl();
                result.setModificationType(objectLocation.getModificationType());
                locationProp.setModificationResult(result);
                aList.add(locationProp);
            }
        }
        
        if ( objectOrientation != null ) {
            if ( objectOrientation.getModificationType() == null ) {
                if (_logger.isEnabledFor(Logger.WARN))
                    logMessage(".modifyEntitySpatialProperties: NO MODIFICATION TYPE SPECIFIED FOR ORIENTATION: " 
                                + objectOrientation + "\n  FOR ENTITY: " + entityGUID + " NO UPDATE WILL BE MADE!", Logger.WARN);   
            }
            else {
                orientationProp = new PropertyImpl();
                orientationProp.setName(PhysicalAddressConcepts.hasOrientation());
                orientationProp.setValue(objectOrientation.getMyId());
                orientationProp.setPropertyObject(objectOrientation);
                ModificationResultImpl result = new ModificationResultImpl();
                result.setModificationType(objectOrientation.getModificationType());
                orientationProp.setModificationResult(result);
                aList.add(orientationProp);
            }
        }
        
        if ( objectOccupiedSpace != null ) {
            if ( objectOccupiedSpace.getModificationType() == null ) {
                if (_logger.isEnabledFor(Logger.WARN))
                    logMessage(".modifyEntitySpatialProperties: NO MODIFICATION TYPE SPECIFIED FOR BLOCK: " 
                                + objectOccupiedSpace + "\n  FOR ENTITY: " + entityGUID + " NO UPDATE WILL BE MADE!", Logger.WARN);   
            }
            else {
                blockProp = new PropertyImpl();
                blockProp.setName(PlaceConcepts.isEncompassedBy());
                blockProp.setValue(objectOccupiedSpace.getMyId());
                blockProp.setPropertyObject(objectOccupiedSpace);
                ModificationResultImpl result = new ModificationResultImpl();
                result.setModificationType(objectOccupiedSpace.getModificationType() );
                blockProp.setModificationResult(result);
                aList.add(blockProp);
            }
        }
        
        if ( aList != null && (aList.size() > 0) ) { 
            modifyEntityProperties(entityGUID, aList);
            modifyObjectSpatialProperties(entityGUID, 
                                          objectLocation, 
                                          objectOrientation,
                                          objectOccupiedSpace);
        }
    }
   
   /** 
    * Modify spatial properties of the object identified by the given objectID in the SpatialReasoner.
    * Important:  Note that the properties are *not* modified in the KAoS DirectoryService, only in the SpatialReasoner.
    *             If you want to modify properties of a given object in the KAoS Directory service, use method modifyEntitySpatialProperties(),
    *             which will do both, modify in the KAoS DS and the SpatialReasoner.
    *
    * @param objectID       String uniquely identifying object, whose spatial properties will be modified in the SpatialReasoner.
    *
    * @param objectLocation        modified Location3D containing the X,Y,Z coordinates of the entity.
    *
    * @param objectOrientation     modified Orientation3D containing the X,Y,Z radius of the entity.
    *
    * @param objectOccupiedSpace   modified Block3D containing corner locations determining the occupied space of the entity.
    *
    * @throws                   DirectoryFailure if the connection to the SpatialReasoner cannot be established.
    *
    * @throws                   NotRegisteredException if the object, whose properties are to be modified has not been registered 
    *                           in the SpatialReasoner.
    *
    */
   public void modifyObjectSpatialProperties(String objectID, 
                                             Location3D objectLocation, 
                                             Orientation3D objectOrientation,
                                             Block3D objectOccupiedSpace)
    throws DirectoryFailure,
           NotRegisteredException
    {
        SpatialReasoner reasoner = null;
        try {
            CSIFactory.setCurrentTransport(_transportName);
            reasoner = CSIFactory.getSpatialReasoner();
        }
        catch(Exception e) {
            e.printStackTrace();
            throw new DirectoryFailure("\n  Unable to obtain SpatialReasoner for transport: " + _transportName
                                        + "\n   Unable to update properties for object: " + objectID);
        }
        reasoner.updateSpatialObject(objectID,  
                                     objectLocation, 
                                     objectOrientation, 
                                     objectOccupiedSpace);
        
    }
    
    /** 
     * Deregister the entity described by the given AgentDescription from the KAoS Directory Service. 
     *
     * @param entityDescription  AgentDescription of the entity to be deregistered.
     *
     * @throws                   DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                   NotRegisteredException if the entity to be deregistered has not been registered in the DS.
     *
     */
    public void deregisterEntity(AgentDescription entityDescription)
    throws DirectoryFailure,
          NotRegisteredException
    {
         _dirProxy.deregister(entityDescription);
    }
          
    /** 
     * Deregister the entity hosting environment described by the given EntityHostEnvDescription from the KAoS Directory Service. 
     *
     * @param eheDescription     EntityHostEnvDescription of the entity hosting environment to be deregistered.
     *
     * @throws                   DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                   NotRegisteredException if the entity to be deregistered has not been registered in the DS.
     *
     */
    public void deregisterEntityHostingEnv(EntityHostEnvDescription  eheDescription)
    throws DirectoryFailure,
          NotRegisteredException
    {
        System.out.println("    \n\n    TO BE IMPLEMENTED!");
    }

    /** 
     * Create an instance of the KAoSAgentDescription using the given parameters.
     *
     * @param guid              String containing the globally unique identifier for the agent, or  null. If null is passed, then
     *                          the Agent Naming Service will create the identifier.
     *
     * @param nickname          String containing agent's nickname.
     *
     * @param domainNames       Vector containing the names of the domains the agent will become a member of.
     *
     * @param transportName     String specifying the name of the transport the agent wants to use, or null.
     *                          If null is specified, then the Locators will be empty.
     *
     */
    public KAoSAgentDescription createKAoSAgentDescription(String guid,
                                                           String nickname,
                                                           Vector domainNames,
                                                           String transportName,
                                                           JasBean agentInitInfo)
    {
        DefaultKAoSAgentDescription desc = new DefaultKAoSAgentDescription();
        AgentName name  = null;
        
        try{
            name = acquireAgentName(guid);
        }
        catch(Exception e) {
            logMessage(".getKAoSAgentDescription: Unable to obtain AgentName for guid:  "
			            + guid + " nickname: " + nickname + "\n   Requested KAoSAgentDescription for cannot be created!", e, Logger.ERROR);
			return null;
        }
        
        if ( name != null ) {
            desc.setAgentName(name);
        }
        else {
            logMessage(".getKAoSAgentDescription: unable to obtain AgentName for agent guid: " + guid
                        + " nickname: " + nickname, Logger.ERROR);
            return null;
        }
        
        if ( nickname != null )
            desc.setAgentNickname(nickname);
        
        if ( transportName != null ) {
            KAoSServiceRoot sr = null;
            try {
		        sr = KAoSBootstrapper.getKAoSServiceRootForTransport(transportName);
		    }
		    catch(Exception e){
			    logMessage("\n  *** Unable to obtain KAoSServiceRoot for transport:  "
			                + transportName + "\n   Requested KAoSAgentDescription for " 
			                + nickname + " cannot be created!", e, Logger.ERROR);
			    return null;
		    }
		    Locator loc = createLocator(nickname, desc.getEntityNameAsString(), agentInitInfo, transportName);
		    if ( loc == null ) return null;
		    sr.setLocatorForDescription(desc, loc);
        }
          
        if ( domainNames != null ) {
            Enumeration names = domainNames.elements();
            while ( names.hasMoreElements() ) {
                String domainName = (String) names.nextElement();
                desc.addDomainName(domainName);
            }
        }
        if (_logger.isEnabledFor(Logger.INFO))
            logMessage(".getKAoSAgentDescription: returning KAoSAgentDescription: " + desc + "\n", Logger.INFO);
            
        return desc;
    }
    /*****************************************************************************************
       kaos.core.csi.registration.DirectoryRegistration interface implementation - end
    *****************************************************************************************/
    
    /*****************************************************************************************
       kaos.core.csi.registration.DomainRegistration interface implementation - begin
    *****************************************************************************************/
    /** 
     * Register the entity described by the given KAoSEntityDescription in the specified domain. 
     *
     * @param domain             DomainDescription describing the domain, in which to register the given entity.
     *
     * @param entityDesc         KAoSEntityDescription describes the entity to be registered in the given domain.
     *
     * @throws                   DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                   AlreadyRegisteredException if the registering entity has been already registered in the 
     *                           the given domain.
     *
     */
    public void registerEntityInDomain(DomainDescription domain,
                                       KAoSEntityDescription entityDesc)
    throws DirectoryFailure,
           AlreadyRegisteredException
    {
        System.out.println("    \n\n    TO BE IMPLEMENTED!");
    }

    /** 
     * Deregister the entity described by the given KAoSEntityDescription from the specified domain. 
     *
     * @param domain             DomainDescription describing the domain, from which to deregister the given entity.
     *
     * @param entityDesc         KAoSEntityDescription describes the entity to be deregistered from the given domain.
     *
     * @throws                   DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                   NotRegisteredException if the deregistering entity has not been registered in the 
     *                           the given domain.
     *
     */
    public void deregisterEntityFromDomain(DomainDescription domain,
                                           KAoSEntityDescription entityDesc)
    throws DirectoryFailure,
           NotRegisteredException
    {
        System.out.println("    \n\n    TO BE IMPLEMENTED!");
    }

    /*****************************************************************************************
       kaos.core.csi.registration.DomainRegistration interface implementation - end
    *****************************************************************************************/
    
    /*****************************************************************************************
       kaos.core.csi.registration.DomainHierarchyConfiguration interface implementation - begin
    *****************************************************************************************/
    /** 
     * Set as a the parent the given parentDomain for the specified domain. 
     *
     * @param domain             DomainDescription describes the domain, whose parent is to be updated.
     *
     * @param parentDomain       DomainDescription describes the parent domain.
     *
     * @throws                   DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                   HasParentException if the specified domain has already a parent. 
     *                           
     *
     */
    public void setParentForDomain(DomainDescription domain,
                                   DomainDescription parentDomain)
    throws DirectoryFailure,
          HasParentException
    {
        System.out.println("    \n\n    TO BE IMPLEMENTED!");
    }

    /** 
     * Add the childDomain as a child of the specified domain.
     *
     * @param domain             DomainDescription describes the domain, to which the childDomain will be added.
     *
     * @param childDomain        DomainDescription describes the child domain.
     *
     * @throws                   DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                   HasParentException if the specified child domain has already a parent. 
     *                           
     *
     */
    public void addChildToDomain(DomainDescription domain,
                                 DomainDescription childDomain )
    throws DirectoryFailure,
           HasParentException
    {
        System.out.println("    \n\n    TO BE IMPLEMENTED!");
    }
           
    /** 
     * Add the childDomain as a child of the specified domain.
     *
     * @param domain             DomainDescription describes the domain, from which to remove the  child domain.
     *
     * @param childDomain        DomainDescription describes the child domain to be removed.
     *
     * @throws                   DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                   NotRegisteredException if the specified domain does not have the specified child domain as a child.
     *                           
     *
     */
    public void removeChildFromDomain(DomainDescription domain,
                                      DomainDescription childDomain )
    throws DirectoryFailure,
           NotRegisteredException
    {
        System.out.println("    \n\n    TO BE IMPLEMENTED!");
    }

    /*****************************************************************************************
       kaos.core.csi.registration.DomainHierarchyConfiguration interface implementation - begin
    *****************************************************************************************/
    /////////////////////////////   PUBLIC METHODS BEGIN
    
    
    /////////////////////////////   PUBLIC METHODS END
    
    /////////////////////////////   PRIVATE METHODS BEGIN
    /*
     **/
    private void initialize(String transportName)
    throws Exception
    {
        try {
            _dirProxy = KAoSBootstrapper.getKAoSDirectoryProxyForTransport(transportName);
        }
        catch(Exception e) {
            e.printStackTrace();
            throw new Exception("    \n\n    *** RegistrationImpl: unable to obtain a reference the DirectoryService for transport: "
                                + transportName + "!");
        }
        _transportName = transportName;
    }
    
    private AgentName acquireAgentName(String guid)
    throws Exception
    {
        AgentName name = null;
        if ( guid != null ) {
            name = new AcrAgentName(DefaultNamespaces.defaultKAoSInstancesNamespace + guid);
        }
        else {
            AgentNamingService ans = new KAoSAgentNamingService();
            name = ans.createAgentName();
        }
        return name;
    }
    
    private Locator createLocator(String nickname, String id, JasBean agentInitInfo, String transportName)
    {
		String agentTransportName = null;
        if ( nickname != null ) {
            agentTransportName = nickname;
        }
        else {
            agentTransportName = id;
        }
        
        TransportSupport transport = CSIFactory.getTransportSupport();
        return transport.createLocator(agentTransportName, agentInitInfo, transportName);
        
    }
    
    //////////////////////////// PRIVATE METHODS END
    // Utility methods ///////////////////////////////////////////
      
    protected void logMessage(String msg, int logLevel)
    {
        _logger.logMessage(msg, logLevel);
    }
	
    protected void logMessage(String msg, Exception e, int logLevel)
    {
   	    _logger.logMessage(msg, e, logLevel);
    }
    
    
    public static void main(String [] args)
      throws Exception      
    {
        String transport = null;
        if (args.length > 0) {
	        int i = 0;
	        while (i < args.length) {
	            if (args[i].equalsIgnoreCase ("-transport")) {
	                if ((i+1) < args.length) {
    	                transport = args[i+1];
    	                i++;
    	            }
    	            else {
    	                System.out.println ("Must specify Agent transport with -transport.  Valid transpors: corba, grid, tcp.");
    	                System.exit(0);
    	            }
	            }
	            i++;
	        }
	    }

        System.out.println ("\n Will request RegistrationImpl  using transport: " + transport);
        Registration registration = RegistrationImpl.getInstance(transport);
        System.out.println ("\n got registration " + registration + "\n\n");
        synchronized ( Thread.currentThread() ) {
            Thread.currentThread().wait();

        }
   }
}

