/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2005/11/10 18:48:10 $
 */

package kaos.core.csi.request;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.request.ActionFailureException;
import kaos.core.csi.request.FeedbackReceiver;
import kaos.core.service.directory.KAoSEntityDescription;

import javax.agent.service.transport.NotLocatableException;
import javax.agent.service.transport.TransportFailure;

/**
 * Defines the operations to send a request expressed by an ActionInstanceDescription to a given entity
 * identified either by a unique guid or a KAoSEntityDescription.
 * 
 * @author KAoS Team
 */
public interface AIDRequestManager 
{
    /**
	 * This method sends a request to an entity identified by the given receiverID to perform an action described  
	 * by the given ActionInstanceDescription.  Feedback from the action execution is sent to the
	 * given feedbackReceiver.
	 * 
	 * @param		feedbackReceiver	    is the FeedbackReceiver requesting the action.  
	 *							            All feedback from the action execution will be sent to it.
	 * 
	 * @param	    receiverID	            is the String that identifies the entity, which is  
	 *							            requested to perform the action.  The receiverID is the one
	 *							            that the action executor has registered with KAoS.
	 * 
	 * @param	    aid                     is an ActionInstanceDescription that describes the action.  
	 *							            It includes the action name and all action properties and their values.
	 * 
	 * @return		String		            containing the unique id of the requested action. This id may be used
	 *                                      by the FeedbackReceiver to track the status of action execution.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *				                        
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 * 
	 **/
    public String sendAIDRequest(FeedbackReceiver feedbackReceiver, 
							     String receiverID, 
                                 ActionInstanceDescription aid)
		throws ActionFailureException, NotLocatableException, TransportFailure;
		
	/**
	 * This method sends a request to an entity identified by the given receiverDesc to perform an action described  
	 * by the given ActionInstanceDescription.  Feedback from the action execution is sent to the
	 * given feedbackReceiver.
	 * 
	 * @param		feedbackReceiver        is the FeedbackReceiver requesting the action.  
	 *							            All feedback from the action execution will be sent to it.
	 * 
	 * @param	    receiverDesc	        is a KAosEntityDescription of the entity, which is requested
	 *								        to perform the action.
	 * 
	 * @param	    aid                     is an ActionInstanceDescription that describes the action.  
	 *							            It includes the action name and all action properties and their values.
	 * 
	 * @return		String		            containing the unique id of the requested action. This id may be used
	 *                                      by the FeedbackReceiver to track the status of the action execution.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 *				                        
	 **/
    public String sendAIDRequest(FeedbackReceiver feedbackReceiver,
								 KAoSEntityDescription receiverDesc,
								 ActionInstanceDescription aid)
		throws ActionFailureException, NotLocatableException, TransportFailure;

		
	/**
	 * This method sends a request described by an ActionInstanceDescription to perform an action by an entity identified 
	 * by the receiverID.
	 * The result is returned synchronously to the caller.
	 *
	 * @param	    receiverID	            is the String that identifies the entity, which is  
	 *							            requested to perform the action.  The receiverID is the one
	 *							            that the action executor has registered with KAoS.
	 *
	 * @param	    aid                     is an ActionInstanceDescription that describes the action.  
	 *							            It includes the action name and all action properties and their values.
	 * 
	 * @return		Object		            containing the final result of the action.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 
	 **/
    public Object sendAIDRequest(String receiverID, 
                                 ActionInstanceDescription aid)
        throws ActionFailureException, NotLocatableException, TransportFailure;
        
    /**
	 * This method sends a request described by an ActionInstanceDescription to perform an action by an entity identified 
	 * by the KAoSEntityDescription.
	 * The result is returned synchronously to the caller.
	 *
	 * @param	    receiverDesc	        is the KAoSEntityDescription that identifies the entity, which is  
	 *							            requested to perform the action.  
	 *
	 * @param	    aid                     is an ActionInstanceDescription that describes the action.  
	 *							            It includes the action name and all action properties and their values.
	 * 
	 * @return		Object		            containing the final result of the action.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 
	 **/
    public Object sendAIDRequest(KAoSEntityDescription receiverDesc, 
                                 ActionInstanceDescription aid)
        throws ActionFailureException, NotLocatableException, TransportFailure;


}
