/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2005/11/28 15:32:56 $
 */
package kaos.core.csi.usecase;

import kaos.core.csi.CSIFactory;
import kaos.core.csi.request.FeedbackReceiver;
import kaos.core.csi.request.RequestManager;
import kaos.core.csi.query.QueryRegistration;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.core.service.directory.KAoSEntityDescription;
import kaos.ontology.vocabulary.ActionConcepts;
import kaos.ontology.vocabulary.ActorConcepts;

import java.util.Calendar;
import java.util.List;
import java.util.Hashtable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

/**
 * $Revision: 1.4 $
 */
public class RequestSenderUsecase
    implements  FeedbackReceiver
 {
    /**
        This class illustrates an implementation of an entity sending a request for an action.
      
        Configuration data includes: nickname, and name of a transport.
        
        Implementation:
            - implements FeedbackReceiver to receive feedback from various stages of processing a requested action.
            
        Bootstrapping steps:
            - obtains a reference to CSI Query
            - obtains a reference to CSI RequestManager

        Running:
            - uses CSI Query to lookup all robots
            - sends various types of requests for CommunicationAction
                async with destination
                async without a specific destination
                sync with a destination
                sync without a specific destination
            - receives results from action execution via the FeedbackReceiver methods

    */

    private RequestManager              _requestMgr;
    private Hashtable                   _actionRequests;
    protected String                    _nickname;
    protected QueryRegistration         _query;
    
    public RequestSenderUsecase( )
    {
    }

    ///////////////////////////////////////////////////////////////////////  
    //  kaos.core.csi.command.FeedbackReceiver implementation
    /////////////////////////////////////////////////////////////////////// 
    /** 
    * This method is invoked when the requested action is accepted by the robot.
    *
    * @param time           Calendar time of acceptance.
    *
    * @param acceptInfo     Object containing additional (optional) info about the acceptance.
    *
    * @param actionID       String containing the id of the requested action/information.
    *
    * @param entityID        String containing the name of the entity performing the action.
    *                       
    */
   public void  onAccepted(Calendar time, Object acceptInfo, String actionID, String entityID)
   {
        String actionName = "Did not find the action for this acceptInfo!!!";
        if (  _actionRequests.containsKey(actionID) ) {
            actionName = (String) _actionRequests.get(actionID);
        }
        System.out.println("\n\n ****   RequestSenderUsecase." + _nickname + ".onAccepted:\n\n    acceptInfo: " + acceptInfo 
                           + "\n    actionID: " + actionID + "\n    actionName: " + actionName + "\n    from entity: " 
                           + entityID +  "\n\n");
   }
   
   /** 
    * This method is invoked when the requested action is refused by the entity performing the action.
    *
    * @param time           Calendar time of refusal.
    *
    * @param reason         Object containing the reason for refusing the request.
    *
    * @param actionID       String containing the id of the requested action/information.
    *
    * @param entityID       String containing the name of the entity performing the action.
    */
   public void onRefused(Calendar time, Object reason, String actionID, String entityID)
   {
        String actionName = "Did not find the action for this result!!!";
        if (  _actionRequests.containsKey(actionID) ) {
            actionName = (String) _actionRequests.get(actionID);
           _actionRequests.remove(actionID); 
        }
        System.out.println("\n\n ****   RequestSenderUsecase." + _nickname + ".onRefused:\n\n    reason: " + reason 
                            + "\n    actionID: " + actionID + "\n    actionName: " + actionName 
                            + "\n    from entity: " + entityID + "\n\n");
   }
   
   /** 
    * This method is invoked upon completion of the request. 
    *
    * @param time           Calendar time of information/action completion.
    *
    * @param result         Object containing the result of the requested action. 
    *
    * @param actionID       String containing the id of the requested action/information.
    *
    * @param entityID       String containing the name of the entity performing the action.
    *
    */
   public void onCompleted(Calendar time, Object result, String actionID, String entityID)
   {
        String actionName = "Did not find the action for this result!!!";
        if (  _actionRequests.containsKey(actionID) ) {
            actionName = (String) _actionRequests.get(actionID);
            _actionRequests.remove(actionID); 
        }
        System.out.println("\n\n ****   RequestSenderUsecase." + _nickname + ".onCompleted:\n\n    result: " + result 
                           + "\n    actionID: " + actionID + "\n    actionName: " + actionName 
                           + "\n    from entity: " + entityID + "\n\n");
   }
   
   /** 
    * This method is invoked when the entity starts the requested action/command.
    *
    * @param time           Calendar time of starting an action.
    *
    * @param startedInfo    Object containing additional (optional) info about the start of the action.
    *
    * @param actionID       String containing the id of the requested action/information.
    *
    * @param entityID       String containing the name of the entity performing the action.
    *
    *                       
    */
   public void onStarted(Calendar time, Object startedInfo, String actionID, String entityID)
   {
        String actionName = "Did not find the action for this result!!!";
        if (  _actionRequests.containsKey(actionID) ) {
            actionName = (String) _actionRequests.get(actionID);
        }
        System.out.println("\n\n ****   RequestSenderUsecase." + _nickname + ".onStarted:\n\n    startedInfo: " + startedInfo 
                           + "\n    actionID: " + actionID + "\n    actionName: " + actionName + "\n    from entity: " 
                           + entityID +  "\n\n");
   }
   
   /** 
    * This method is invoked when the requested action is aborted.
    *
    * @param time           Calendar time of aborting an action.
    *
    * @param reason         Object containing the reason for aborting an action.
    *
    * @param actionID       String containing the id of the requested action.
    *
    * @param entityID       String containing the name of the entity performing the action.
    *
    */
    public void onAborted(Calendar time, Object reason, String actionID, String entityID)
    {
        String actionName = "Did not find the action for this reason!!!";
        if (  _actionRequests.containsKey(actionID) ) {
            actionName = (String) _actionRequests.get(actionID);
           _actionRequests.remove(actionID); 
        }
        System.out.println("\n\n ****   RequestSenderUsecase." + _nickname + ".onAborted:\n\n    reason: " + reason 
                           + "\n    actionID: " + actionID + "\n    actionName: " + actionName + "\n    from entity: " 
                           + entityID +  "\n\n");    }
   
    /**
     * Receive a status message describing the execution of the requested action. 
     * There maybe several status messages sent during action execution.
     *
     * @param statusMsg     Object containing the result of the requested action/information.
     *
     * @param actionID      String containing the id of the requested action/information.
     *
     * @param entityID       String containing the name of the entity performing the action.
     *
     */                                     
    public void receiveKAoSStatus(String statusMsg, String actionID, String entityID)
    {
     System.out.println("\n\n ****   RequestSenderUsecase.receiveKAoSStatus: received statusMsg: " + statusMsg 
                            + " actionID: " + actionID + " from entity: " + entityID +  "\n\n");
    }
   
    ///////////////////////////////////////////////////////////////////////  
    //  kaos.core.csi.command.FeedbackReceiver implementation - end
    /////////////////////////////////////////////////////////////////////// 
 
    //////////////////////////////////////////////////////////////

    public void initialize(String name,  String transportName)
    throws Exception
    {
        _nickname = name;
        CSIFactory.setCurrentTransport(transportName);
        _query = CSIFactory.getQueryRegistration();
        _requestMgr = CSIFactory.getRequestManager();
        _actionRequests = new Hashtable();
        
        System.out.println("\n   RequestSenderUsecase: initialized!!" );
    }
    
    public void test()
    {
        /*
            Find all agents registered in the KAoS Directory Service containing in their KAoSAgentDescriptions Agent and Robot
            ontological types.
        */
        Hashtable params = new Hashtable(1);
        Vector ontNames = new Vector();
        ontNames.add(ActorConcepts.Agent());
        ontNames.add(ActorConcepts.Robot());
      
        params.put(KAoSEntityDescription.ENTITY_ONTOLOGICAL_TYPE_NAMES,ontNames);
        System.out.println("\n  Sending search request with params: \n" + params);
      
        List results = null;
        try {
            results = _query.getActorsWithAttributes(params);
            System.out.println("\n  Results for entity search: \n");
            Iterator foundRobots = results.iterator();
            while (foundRobots.hasNext()) {
                KAoSAgentDescription entity = (KAoSAgentDescription)foundRobots.next();
                System.out.println("\n      " + entity.getEntityNameAsString() + " nickname: " + entity.getAgentNickname());
            }
            System.out.println("\n");
        }
        catch(Exception e){
            e.printStackTrace();
            System.out.println("\n  Robot search failed, unable to send commands!");
            return;
        }
        
        Iterator foundRobots = results.iterator();
        while (foundRobots.hasNext()) {
            KAoSAgentDescription entity = (KAoSAgentDescription)foundRobots.next();
            testRequestsTo(entity);
        }
    }
    
   private void testRequestsTo(KAoSAgentDescription robot)
   {
        ////////////////// CommunicationAction request 
        // Test 1: asynchronous request passing properties and a feedback receiver.  Result and status from the various stages
        // of action execution are sent via the feeedback's API.
        HashMap properties = new HashMap();
       
        properties.put(ActionConcepts.hasDestination(), "urn#KAoS:usecase"); 
        try {
            String requestID1 = _requestMgr.sendHashMapRequest(this,
                                                               robot,
                                                               ActionConcepts.CommunicationAction(),
                                                               properties);
            _actionRequests.put(requestID1, ActionConcepts.CommunicationAction());
        }
        catch(Exception e) {
            System.out.println("\n  **** Request 1 for CommunicationAction with destination urn#KAoS:usecase failed with the following exception: \n");
            e.printStackTrace();
        }
        
        // Test 2: asynchronous request passing a feedback receiver. No properties are specified. Result and status from 
        // the various stages of action execution are sent via the feeedback's API.        
        try {
            String requestID2 = _requestMgr.sendHashMapRequest(this,
                                                               robot,
                                                               ActionConcepts.CommunicationAction());
            _actionRequests.put(requestID2, ActionConcepts.CommunicationAction());
        }
        catch(Exception e) {
            e.printStackTrace();
            System.out.println("\n  **** Request 2 for CommunicationAction with no properties failed with the following exception: \n");
        }
        
        // Test 3: synchronous request.  Action properties are specified. Final result from action execution is returned. 
        //         
        properties.put(ActionConcepts.hasDestination(), "urn#KAoS:usecase-BLOCKING"); 
        try {
            Object result1 = _requestMgr.sendHashMapRequest(robot,
                                                            ActionConcepts.CommunicationAction(),
                                                            properties);
            System.out.println("\n  Sync Request 1 (with properties) for CommunicationAction returned: " + result1);
        }
        catch(Exception e) {
             System.out.println("\n     **** Request for sync CommunicationAction with destination urn#KAoS:usecase-BLOCKING failed with the following exception: \n");
             e.printStackTrace();
        }
        
        // Test 4: synchronous request.  No action properties are specified. Final result from action execution is returned. 
        //         
        try {
                                              
            Object result2 = _requestMgr.sendHashMapRequest(robot,
                                                            ActionConcepts.CommunicationAction());
           System.out.println("\n  Sync Request 2 (without properties) for CommunicationAction returned: " + result2);
        }
        catch(Exception e) {
             System.out.println("\n     **** Request  for sync CommunicationAction without properties failed with the following exception: \n");
             e.printStackTrace();
        }
        
        ////////////////// MonitorAction request 
        // Test 5: synchronous request.  No action properties are specified. Final result from action execution is returned. 
        //         
        try {
                                              
            _requestMgr.sendHashMapRequest(robot,
                                           ActionConcepts.MonitorAction());
           System.out.println("\n  Request 3 for MonitorAction(without properties).") ;
        }
        catch(Exception e) {
             e.printStackTrace();
        }
        System.out.println("\n  No more requests...") ;
   }
    
   //////////////////////////////////////////////////////////////
   // Entry point...

   public static void main(String [] args)
      throws Exception      
    {
        String name = null;
        String transport = null;
        if (args.length > 0) {
	        int i = 0;
	        while (i < args.length) {
	            if (args[i].equalsIgnoreCase ("-name")) {
	                if ((i+1) < args.length) {
    	                name = args[i+1];
    	                i++;
    	            }
    	            else {
    	                System.out.println ("Must specify sender name with -name");
    	                System.exit(0);
    	            }
	            }
	            else if (args[i].equalsIgnoreCase ("-transport")) {
	                if ((i+1) < args.length) {
    	                transport = args[i+1];
    	                i++;
    	            }
    	            else {
    	                System.out.println ("Must specify  transport with -transport.  Valid transports: corba, grid, tcp.");
    	                System.exit(0);
    	            }
	            }
                 i++;
	        }
	    }
	    
        if ( name == null) {
            System.out.println ("\n     *** Must specify sender name with -name\n");
            return;
        }
        
        if ( transport == null) {
            System.out.println ("\n     *** Must specify  transport with -transport.  Valid transports: corba, grid, tcp.");
            return;
        }
                
        System.out.println ("\n     <>  Name of sender:             " + name);
	    System.out.println ("\n     <>  Requested transport:        " + transport);
	    	                   
        RequestSenderUsecase usecase = new RequestSenderUsecase();
        
        try {
            usecase.initialize(name, transport);
            usecase.test();
        }
        catch(Exception e) {
            e.printStackTrace();
            return;
           
        }
        
        synchronized ( Thread.currentThread() ) {
         Thread.currentThread().wait();

      }

   }
}

